# -*- coding: utf-8 -*-
# Copyright (C) Duncan Macleod (2013)
#
# This file is part of gwemopt
#
# gwemopt is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gwemopt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gwemopt.  If not, see <http://www.gnu.org/licenses/>

"""Git version generator
"""

__author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
__credits__ = 'Adam Mercer <adam.mercer@ligo.org>'

import os
import subprocess
import time

try:
    unicode = unicode
except NameError:
    # 'unicode' is undefined, must be Python 3
    str = str
    unicode = str
    bytes = bytes
    basestring = (str,bytes)
else:
    # 'unicode' exists, must be Python 2
    str = str
    unicode = unicode
    bytes = str
    basestring = basestring

class GitStatus(object):
    """Git repository version information
    """
    def __init__(self):
        self._bin = self._find_git()
        self.id = None
        self.date = None
        self.branch = None
        self.tag = None
        self.author = None
        self.committer = None
        self.status = None

    # ------------------------------------------------------------------------
    # Core methods

    @staticmethod
    def _find_git():
        """Determine the full path of the git binary on this
        host
        """
        for path in os.environ['PATH'].split(os.pathsep):
            gitbin = os.path.join(path, 'git')
            if os.path.isfile(gitbin) and os.access(gitbin, os.X_OK):
                return gitbin
        raise ValueError("Git binary not found on this host")

    def git(self, *args):
        """Executable a command with arguments in a sub-process
        """
        cmdargs = [self._bin] + list(args)
        p = subprocess.Popen(cmdargs,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE,
                             shell=isinstance(args, basestring))
        out, err = p.communicate()
        if p.returncode != 0:
            raise subprocess.CalledProcessError(p.returncode,
                                                ' '.join(cmdargs))
        return out.strip()

    # ------------------------------------------------------------------------
    # Git communication methods

    def get_commit_info(self):
        """Determine basic info about the latest commit
        """
        a, b, c, d, e, f =  self.git(
            'log', '-1', '--pretty=format:%H,%ct,%an,%ae,%cn,%ce').decode().split(',')
        self.id = a
        self.udate = b
        author = c
        author_email = d
        committer = e
        committer_email = f
        self.date = time.strftime('%Y-%m-%d %H:%M:%S +0000',
                                  time.gmtime(float(self.udate)))
        self.author = '%s <%s>' % (author, author_email)
        self.committer = '%s <%s>' % (committer, committer_email)

    def get_branch(self):
        branch = self.git('rev-parse', '--symbolic-full-name', 'HEAD').decode('utf-8')
        if branch == 'HEAD':
            self.branch = None
        else:
            self.branch = os.path.basename(branch)
        return self.branch

    def get_status(self):
        """Determine modification status of working tree
        """
        try:
            status = self.git('diff-files', '--quiet')
        except subprocess.CalledProcessError:
            self._status = 'UNCLEAN: Modified working tree'
        else:
            try:
                status = self.git('diff-index', '--cache', '--quiet',
                                  'HEAD')
            except subprocess.CalledProcessError:
                self.status = 'UNCLEAN: Modified working tree'
            else:
                self.status = 'CLEAN: All modifications committed'
        return self.status

    def get_tag(self):
        """Determine name of the current tag
        """
        if not self.id:
            self.get_commit_info()
        try:
            self.tag = self.git('describe', '--exact-match', '--tags',
                                 self.id)
        except subprocess.CalledProcessError:
            self.tag = None
        return self.tag

    # ------------------------------------------------------------------------
    # Write

    def write(self, fobj):
        """Write the contents of this `GitStatus` to a version.py format
        file object
        """
        # write file header
        fobj.write("# -*- coding: utf-8 -*-\n"
                   "# Copyright (C) Duncan Macleod (2013)\n\n"
                   "\"\"\"Versioning record for CIS\n\"\"\"\n\n")

        # write standard pythonic metadata
        fobj.write("__author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'\n"
                   "__version__ = '%s'\n"
                   "__date__ = '%s'\n\n" % (self.version, self.date))

        # write git information
        for attr in ['id', 'branch', 'tag', 'author', 'committer', 'status']:
            val = getattr(self, attr)
            if val:
                fobj.write("git_%s = '%s'\n" % (attr, val))
            else:
                fobj.write("git_%s = None\n" % attr)

    def __call__(self, outputfile='version.py'):
        """Process the version information into a new file

        Parameters
        ----------
        outputfile : `str`
            path to output python file in which to write version info

        Returns
        -------
        info : `str`
            returns a string dump of the contents of the outputfile
        """
        self.get_commit_info()
        self.get_branch()
        self.get_tag()
        self.get_status()
        self.version = self.tag or self.id
        with open(outputfile, 'w') as fobj:
            self.write(fobj)
        with open(outputfile, 'r') as fobj:
            return fobj.read()




import os, sys, copy
import numpy as np
import healpy as hp

from scipy.stats import norm

import matplotlib
#matplotlib.rc('text', usetex=True)
matplotlib.use('Agg')
matplotlib.rcParams.update({'font.size': 16})
matplotlib.rcParams['contour.negative_linestyle'] = 'solid'
import matplotlib.pyplot as plt

def observability(params,map_struct):
    observability_struct = map_struct["observability"]

    unit='Gravitational-wave probability'
    cbar=False

    for telescope in observability_struct.keys():
        plotName = os.path.join(params["outputDir"],'observability_%s.pdf'%telescope)
        hp.mollview(map_struct["prob"]*observability_struct[telescope]["observability"],title='',unit=unit,cbar=cbar,min=np.min(map_struct["prob"]),max=np.max(map_struct["prob"]))
        add_edges()
        plt.show()
        plt.savefig(plotName,dpi=200)
        plt.close('all')

    if params["doMovie"]:
        moviedir = os.path.join(params["outputDir"],'movie')
        if not os.path.isdir(moviedir): os.mkdir(moviedir)
    
        for telescope in observability_struct.keys():
            dts = observability_struct[telescope]["dts"].keys()
            dts = np.sort(dts)
            for ii,dt in enumerate(dts):
                plotName = os.path.join(moviedir,'observability-%04d.png'%ii)
                title = "Detectability Map: %.2f Days"%dt
                hp.mollview(map_struct["prob"]*observability_struct[telescope]["dts"][dt],title=title,cbar=cbar,min=np.min(map_struct["prob"]),max=np.max(map_struct["prob"]))
                add_edges()
                plt.show()
                plt.savefig(plotName,dpi=200)
                plt.close('all')
    
            moviefiles = os.path.join(moviedir,"observability-%04d.png")
            filename = os.path.join(params["outputDir"],"observability_%s.mpg"%telescope)
            ffmpeg_command = 'ffmpeg -an -y -r 20 -i %s -b:v %s %s'%(moviefiles,'5000k',filename)
            os.system(ffmpeg_command)
            filename = os.path.join(params["outputDir"],"observability_%s.gif"%telescope)
            ffmpeg_command = 'ffmpeg -an -y -r 20 -i %s -b:v %s %s'%(moviefiles,'5000k',filename)
            os.system(ffmpeg_command)
            rm_command = "rm %s/*.png"%(moviedir)
            os.system(rm_command)
    
def tauprob(params,tau,prob):

    plotName = os.path.join(params["outputDir"],'tau_prob.pdf')
    plt.figure()
    plt.plot(tau, prob)
    plt.grid()
    plt.xscale('log')
    plt.yscale('log')
    plt.xlabel('Log of observing time $\tau$',fontsize = 20)
    plt.ylabel('Log of detection prob. given the target is at the observing field',fontsize = 20)
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def tiles(params,map_struct,tiles_structs):

    unit='Gravitational-wave probability'
    cbar=False

    plotName = os.path.join(params["outputDir"],'tiles.pdf')
    plt.figure()
    hp.mollview(map_struct["prob"],title='',unit=unit,cbar=cbar)
    ax = plt.gca()
    for telescope in tiles_structs:
        tiles_struct = tiles_structs[telescope]
        for index in tiles_struct.keys():
            ipix, corners, patch = tiles_struct[index]["ipix"], tiles_struct[index]["corners"], tiles_struct[index]["patch"]
            #hp.visufunc.projplot(corners[:,0], corners[:,1], 'k', lonlat = True)
            if not patch: continue
            hp.projaxes.HpxMollweideAxes.add_patch(ax,patch)
            #tiles.plot()
    add_edges()
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

    if params["doReferences"]:

        rot=(90,0,0)
        plotName = os.path.join(params["outputDir"],'tiles_ref.pdf')
        plt.figure()
        hp.mollview(np.zeros(map_struct["prob"].shape),title='',unit=unit,cbar=cbar,cmap=plt.get_cmap('Greens'))
        ax = plt.gca()
        for telescope in tiles_structs:
            config_struct = params["config"][telescope]
            tiles_struct = tiles_structs[telescope]
            for index in tiles_struct.keys():
                if not index in config_struct["reference_images"]: continue
                if len(params["filters"]) == 1:
                    if not params["filters"][0] in config_struct["reference_images"][index]:
                        continue
                ipix, corners, patch = tiles_struct[index]["ipix"], tiles_struct[index]["corners"], tiles_struct[index]["patch"]
                #hp.visufunc.projplot(corners[:,0], corners[:,1], 'k', lonlat = True)
                if not patch: continue
                patch_cpy = copy.copy(patch)
                patch_cpy.axes = None
                patch_cpy.figure = None
                patch_cpy.set_transform(ax.transData)
                hp.projaxes.HpxMollweideAxes.add_patch(ax,patch_cpy)
                #tiles.plot()
        add_edges()
        plt.show()
        plt.savefig(plotName,dpi=200)
        plt.close('all')

def add_edges():

    hp.graticule(verbose=False)
    plt.grid(True)
    lons = np.arange(-150.0,180,30.0)
    lats = np.zeros(lons.shape)
    for lon, lat in zip(lons,lats):
        hp.projtext(lon,lat,"%.0f"%lon,lonlat=True)
    lats = np.arange(-60.0,90,30.0)
    lons = np.zeros(lons.shape)
    for lon, lat in zip(lons,lats):
        hp.projtext(lon,lat,"%.0f"%lat,lonlat=True)

def skymap(params,map_struct):

    unit='Gravitational-wave probability'
    cbar=False

    lons = np.arange(-150.0,180,30.0)
    lats = np.zeros(lons.shape)

    plotName = os.path.join(params["outputDir"],'prob.pdf')
    hp.mollview(map_struct["prob"],title='',unit=unit,cbar=cbar,min=np.percentile(map_struct["prob"],1),max=np.percentile(map_struct["prob"],99))
    add_edges()
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

    if "distmu" in map_struct:
        plotName = os.path.join(params["outputDir"],'dist.pdf')
        hp.mollview(map_struct["distmu"],unit='Distance [Mpc]',min=np.percentile(map_struct["distmu"],10),max=np.percentile(map_struct["distmu"],90))
        add_edges()
        plt.show()
        plt.savefig(plotName,dpi=200)
        plt.close('all')

        plotName = os.path.join(params["outputDir"],'dist_median.pdf')
        hp.mollview(map_struct["distmed"],unit='Distance [Mpc]',min=np.percentile(map_struct["distmed"],10),max=np.percentile(map_struct["distmed"],90))
        add_edges()
        plt.show()
        plt.savefig(plotName,dpi=200)
        plt.close('all')

def waw(params, detmaps, t_detmaps, strategy_struct):

    if params["doMovie"]:
        moviedir = os.path.join(params["outputDir"],'movie')
        if not os.path.isdir(moviedir): os.mkdir(moviedir)
        
        for ii in range(len(t_detmaps)):
            t_detmap = t_detmaps[ii]
            detmap = detmaps[ii]

            plotName = os.path.join(moviedir,'detmap-%04d.png'%ii)
            title = "Detectability Map: %.2f Days"%t_detmap
            hp.mollview(detmap,title=title,min=0.0,max=1.0,unit="Probability of Detection")
            add_edges()
            plt.show()
            plt.savefig(plotName,dpi=200)
            plt.close('all')

        moviefiles = os.path.join(moviedir,"detmap-%04d.png")
        filename = os.path.join(params["outputDir"],"detmap.mpg")
        ffmpeg_command = 'ffmpeg -an -y -r 20 -i %s -b:v %s %s'%(moviefiles,'5000k',filename)
        os.system(ffmpeg_command)
        filename = os.path.join(params["outputDir"],"detmap.gif")
        ffmpeg_command = 'ffmpeg -an -y -r 20 -i %s -b:v %s %s'%(moviefiles,'5000k',filename)
        os.system(ffmpeg_command)    
        rm_command = "rm %s/*.png"%(moviedir)
        os.system(rm_command)

def efficiency(params, map_struct, efficiency_structs):

    unit='Gravitational-wave probability'
    cbar=False

    plotName = os.path.join(params["outputDir"],'efficiency.pdf')
    plt.figure()
    for key in efficiency_structs:
        efficiency_struct = efficiency_structs[key]
        plt.plot(efficiency_struct["distances"],efficiency_struct["efficiency"],label=efficiency_struct["legend_label"])
    plt.xlabel('Distance [Mpc]')
    plt.ylabel('Efficiency')
    plt.legend(loc="best")
    plt.ylim([0.01,1])
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

    plotName = os.path.join(params["outputDir"],'injs.pdf')
    plt.figure()
    plt.plot(efficiency_struct["ra"],efficiency_struct["dec"],'kx')
    plt.xlabel('RA [Degrees]')
    plt.ylabel('Declination [Degrees]')
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

    plotName = os.path.join(params["outputDir"],'mollview_injs.pdf')
    plt.figure()
    hp.mollview(map_struct["prob"],unit=unit,cbar=cbar)
    hp.projplot(efficiency_struct["ra"], efficiency_struct["dec"], 'wx', lonlat=True, coord='G')
    add_edges()
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def coverage(params, map_struct, coverage_struct):

    unit='Gravitational-wave probability'
    cbar=False

    plotName = os.path.join(params["outputDir"],'mollview_coverage.pdf')
    plt.figure()
    hp.mollview(map_struct["prob"],title='',unit=unit,cbar=cbar)
    hp.projplot(coverage_struct["data"][:,0], coverage_struct["data"][:,1], 'wx', lonlat=True, coord='G')
    add_edges()
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

    idx = np.isfinite(coverage_struct["data"][:,4])
    if not idx.size: return
    min_time = np.min(coverage_struct["data"][idx,4])
    max_time = np.max(coverage_struct["data"][idx,4])

    plotName = os.path.join(params["outputDir"],'tiles_coverage.pdf')
    plt.figure()
    hp.mollview(map_struct["prob"],title='',unit=unit,cbar=cbar)
    add_edges()
    ax = plt.gca()
    for ii in range(len(coverage_struct["ipix"])):
        data = coverage_struct["data"][ii,:]
        filt = coverage_struct["filters"][ii]
        ipix = coverage_struct["ipix"][ii]
        patch = coverage_struct["patch"][ii]
        FOV = coverage_struct["FOV"][ii]

        #hp.visufunc.projplot(corners[:,0], corners[:,1], 'k', lonlat = True)
        patch_cpy = copy.copy(patch)
        patch_cpy.axes = None
        patch_cpy.figure = None
        patch_cpy.set_transform(ax.transData)
        hp.projaxes.HpxMollweideAxes.add_patch(ax,patch_cpy)
        #tiles.plot()
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

    plotName = os.path.join(params["outputDir"],'tiles_coverage_scaled.pdf')
    plt.figure()
    hp.mollview(map_struct["prob"],title='',unit=unit,cbar=cbar)
    add_edges()
    ax = plt.gca()
    for ii in range(len(coverage_struct["ipix"])):
        data = coverage_struct["data"][ii,:]
        filt = coverage_struct["filters"][ii]
        ipix = coverage_struct["ipix"][ii]
        patch = coverage_struct["patch"][ii]
        FOV = coverage_struct["FOV"][ii]

        #hp.visufunc.projplot(corners[:,0], corners[:,1], 'k', lonlat = True)
        patch_cpy = copy.copy(patch)
        patch_cpy.axes = None
        patch_cpy.figure = None
        patch_cpy.set_transform(ax.transData)
        current_alpha = patch_cpy.get_alpha()

        if current_alpha > 0.0:
            alpha = data[4]/max_time
            if alpha > 1:
                alpha = 1.0
            patch_cpy.set_alpha(alpha)
        hp.projaxes.HpxMollweideAxes.add_patch(ax,patch_cpy)
        #tiles.plot()
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

    if params["doMovie"]:
        idx = np.isfinite(coverage_struct["data"][:,2])
        mjd_min = np.min(coverage_struct["data"][idx,2])
        mjd_max = np.max(coverage_struct["data"][idx,2])
        mjd_N = 100
    
        mjds = np.linspace(mjd_min,mjd_max,num=mjd_N)
        moviedir = os.path.join(params["outputDir"],'movie')
        if not os.path.isdir(moviedir): os.mkdir(moviedir)
    
        #for jj in range(len(coverage_struct["ipix"])):
        #    mjd = coverage_struct["data"][jj,3]
        for jj in range(len(mjds)):
            mjd = mjds[jj]
            plotName = os.path.join(moviedir,'coverage-%04d.png'%jj)
            title = "Coverage Map: %.2f"%mjd       
    
            plt.figure()
            hp.mollview(map_struct["prob"],title=title,unit=unit,cbar=cbar)
            add_edges()
            ax = plt.gca()
    
            idx = np.where(coverage_struct["data"][:,2]<=mjd)[0]
            #for ii in range(jj):
            for ii in idx: 
                data = coverage_struct["data"][ii,:]
                filt = coverage_struct["filters"][ii]
                ipix = coverage_struct["ipix"][ii]
                patch = coverage_struct["patch"][ii]
                FOV = coverage_struct["FOV"][ii]
    
                #hp.visufunc.projplot(corners[:,0], corners[:,1], 'k', lonlat = True)
                patch_cpy = copy.copy(patch)
                patch_cpy.axes = None
                patch_cpy.figure = None
                patch_cpy.set_transform(ax.transData)
                #alpha = data[4]/max_time
                #if alpha > 1:
                #    alpha = 1.0
                #patch_cpy.set_alpha(alpha)
                hp.projaxes.HpxMollweideAxes.add_patch(ax,patch_cpy)
                #tiles.plot()
            plt.show()
            plt.savefig(plotName,dpi=200)
            plt.close('all')
            
        moviefiles = os.path.join(moviedir,"coverage-%04d.png")
        filename = os.path.join(params["outputDir"],"coverage.mpg")
        ffmpeg_command = 'ffmpeg -an -y -r 20 -i %s -b:v %s %s'%(moviefiles,'5000k',filename)
        os.system(ffmpeg_command)
        filename = os.path.join(params["outputDir"],"coverage.gif")
        ffmpeg_command = 'ffmpeg -an -y -r 20 -i %s -b:v %s %s'%(moviefiles,'5000k',filename)
        os.system(ffmpeg_command)
        rm_command = "rm %s/*.png"%(moviedir)
        os.system(rm_command)

def scheduler(params,exposurelist,keys): 
    plotName = os.path.join(params["outputDir"],'scheduler.pdf')
    if params['scheduleType'].endswith('_slew'):
        e = []
        k = []
        start_time = exposurelist[0][0]
        for exposure, key in zip(exposurelist, keys):
            e.append((exposure[0] - start_time) * 24)
            k.append(key)
            e.append((exposure[1] - start_time) * 24)
            k.append(key)
        plt.figure()
        plt.grid()
        plt.xlabel("Time (h)")
        plt.ylabel("Tile Number")
        plt.plot(e, k, 'b-')
    else:
        xs = []
        ys = []
        for ii,key in zip(np.arange(len(exposurelist)),keys):
            xs.append(ii)
            ys.append(key)    
        plt.figure()
        plt.xlabel("Exposure Number")
        plt.ylabel("Tile Number")
        plt.plot(xs,ys,'kx')
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')

def transients(params, map_struct, transients_struct):

    unit='Gravitational-wave probability'
    cbar=False

    ra = transients_struct["data"][:,0]
    dec = transients_struct["data"][:,1]

    plotName = os.path.join(params["outputDir"],'transients.pdf')
    plt.figure()
    hp.mollview(map_struct["prob"],unit=unit,cbar=cbar)
    hp.projplot(ra, dec, 'wx', lonlat=True, coord='G')
    add_edges()
    plt.show()
    plt.savefig(plotName,dpi=200)
    plt.close('all')





import os, sys, copy
import h5py
import numpy as np
import healpy as hp

from scipy.stats import norm
from scipy.special import gammaincinv

from astroquery.vizier import Vizier
Vizier.ROW_LIMIT = -1

from astropy.table import Table
from astropy.io import ascii
from astropy.cosmology import WMAP9 as cosmo
from astropy.table import Column
import astropy.units as u
import astropy.constants as c

def get_catalog(params, map_struct):

    if not os.path.isdir(params["catalogDir"]): os.makedirs(params["catalogDir"])
    catalogFile = os.path.join(params["catalogDir"],"%s.hdf5"%params["galaxy_catalog"])

    """AB Magnitude zero point."""
    MAB0 = -2.5 * np.log10(3631.e-23)
    pc_cm = 3.08568025e18
    const = 4. * np.pi * (10. * pc_cm)**2.

    if params["galaxy_catalog"] == "2MRS":
        if not os.path.isfile(catalogFile):
            cat, = Vizier.get_catalogs('J/ApJS/199/26/table3')

            ras, decs = cat["RAJ2000"], cat["DEJ2000"]
            cz = cat["cz"]
            magk = cat["Ktmag"]
            z = (u.Quantity(cat['cz']) / c.c).to(u.dimensionless_unscaled)

            completeness = 0.5
            alpha = -1.0
            MK_star = -23.55
            MK_max = MK_star + 2.5 * np.log10(gammaincinv(alpha + 2, completeness))
            MK = magk - cosmo.distmod(z)
            idx = (z > 0) & (MK < MK_max)
        
            ra, dec = ra[idx], dec[idx]
            z = z[idx]
            magk = magk[idx]            

            with h5py.File(catalogFile, 'w') as f:
                f.create_dataset('ra', data=ras)
                f.create_dataset('dec', data=decs)
                f.create_dataset('z', data=z)
                f.create_dataset('magk', data=magk)

        else:
            with h5py.File(catalogFile, 'r') as f:
                ra, dec = f['ra'][:], f['dec'][:]
                z = f['z'][:]
                magk = f['magk'][:]

        r = cosmo.luminosity_distance(z).to('Mpc').value
        mag = magk * 1.0
 
    elif params["galaxy_catalog"] == "GLADE":
        if not os.path.isfile(catalogFile):
            cat, = Vizier.get_catalogs('VII/281/glade2')

            ra, dec = cat["RAJ2000"], cat["DEJ2000"]
            distmpc = cat["Dist"]
            magb, magk = cat["Bmag"], cat["Kmag"]

            idx = np.where(distmpc >= 0)[0]
            ra, dec = ra[idx], dec[idx]
            distmpc = distmpc[idx]
            magb, magk = magb[idx], magk[idx]

            with h5py.File(catalogFile, 'w') as f:
                f.create_dataset('ra', data=ra)
                f.create_dataset('dec', data=dec)
                f.create_dataset('distmpc', data=distmpc)
                f.create_dataset('magb', data=magb)
                f.create_dataset('magk', data=magk)
        else:
            with h5py.File(catalogFile, 'r') as f:
                ra, dec = f['ra'][:], f['dec'][:]
                distmpc = f['distmpc'][:]
                magb, magk = f['magb'][:], f['magk'][:]

        idx = np.where(~np.isnan(magb))[0]
        ra, dec, distmpc, magb, magk = ra[idx], dec[idx], distmpc[idx], magb[idx], magk[idx]

        r = distmpc * 1.0
        mag = magb * 1.0

    elif params["galaxy_catalog"] == "CLU":
        if not os.path.isfile(catalogFile):
            print("Please add %s."%catalogFile)
            exit(0)

        with h5py.File(catalogFile, 'r') as f:
            name = f['name'][:]
            ra, dec = f['ra'][:], f['dec'][:]
            sfr_fuv, mstar = f['sfr_fuv'][:], f['mstar'][:]
            distmpc, magb = f['distmpc'][:], f['magb'][:]
            a, b2a, pa = f['a'][:], f['b2a'][:], f['pa'][:]
            btc = f['btc'][:]

        idx = np.where(distmpc >= 0)[0]
        ra, dec = ra[idx], dec[idx]
        sfr_fuv, mstar = sfr_fuv[idx], mstar[idx]
        distmpc, magb = distmpc[idx], magb[idx]
        a, b2a, pa = a[idx], b2a[idx], pa[idx]
        btc = btc[idx]

        idx = np.where(~np.isnan(magb))[0]
        ra, dec = ra[idx], dec[idx]
        sfr_fuv, mstar = sfr_fuv[idx], mstar[idx]
        distmpc, magb = distmpc[idx], magb[idx]
        a, b2a, pa = a[idx], b2a[idx], pa[idx]
        btc = btc[idx]

        r = distmpc * 1.0
        mag = magb * 1.0

    L_nu =  const * 10.**((mag + MAB0)/(-2.5))
    L_nu = np.log10(L_nu)
    L_nu = L_nu**params["catalog_n"]
    Slum = L_nu / np.sum(L_nu)

    mlim, M_KNmin, M_KNmax = 22, -17, -12
    L_KNmin, L_KNmax = const * 10.**((M_KNmin + MAB0)/(-2.5)), const * 10.**((M_KNmax + MAB0)/(-2.5))

    Llim = 4. * np.pi * (r * 1e6 * pc_cm)**2. * 10.**((mlim + MAB0)/(-2.5))
    Sdet = (L_KNmax-Llim)/(L_KNmax-L_KNmin)
    Sdet[Sdet<0.01] = 0.01
    Sdet[Sdet>1.0] = 1.0

    n, cl, dist_exp = params["powerlaw_n"], params["powerlaw_cl"], params["powerlaw_dist_exp"]
    prob_scaled = copy.deepcopy(map_struct["prob"])
    prob_sorted = np.sort(prob_scaled)[::-1]
    prob_indexes = np.argsort(prob_scaled)[::-1]
    prob_cumsum = np.cumsum(prob_sorted)
    index = np.argmin(np.abs(prob_cumsum - cl)) + 1
    prob_scaled[prob_indexes[index:]] = 0.0
    prob_scaled = prob_scaled**n
#    prob_scaled = prob_scaled / np.nansum(prob_scaled)

    theta = 0.5 * np.pi - dec * 2 * np.pi /360.0
    phi = ra * 2 * np.pi /360.0
    ipix = hp.ang2pix(map_struct["nside"], ra, dec, lonlat=True).astype(int)
    if "distnorm" in map_struct:
        if map_struct["distnorm"] is not None:
            Sloc = prob_scaled[ipix] * (map_struct["distnorm"][ipix] * norm(map_struct["distmu"][ipix], map_struct["distsigma"][ipix]).pdf(r))**params["powerlaw_dist_exp"] / map_struct["pixarea"]
    else:
        Sloc = copy.copy(prob_scaled[ipix])

    S = Sloc*Slum*Sdet

    prob = np.zeros(map_struct["prob"].shape)
    prob[ipix] = prob[ipix] + S
    prob = prob / np.sum(prob)

    map_struct['prob_catalog'] = prob
    if params["doUseCatalog"]:
        map_struct['prob'] = prob 

    idx = np.where(~np.isnan(S))[0]
    ra, dec, Sloc, S = ra[idx], dec[idx], Sloc[idx], S[idx]

    Sthresh = np.max(S)*0.01
    idx = np.where(S >= Sthresh)[0]
    ra, dec, Sloc, S = ra[idx], dec[idx], Sloc[idx], S[idx] 

    idx = np.argsort(S)[::-1]
    ra, dec, Sloc, S = ra[idx], dec[idx], Sloc[idx], S[idx] 

    if len(ra) > 1000:
        print('Cutting catalog to top 1000 galaxies...')
        idx = np.arange(1000).astype(int)
        ra, dec, Sloc, S = ra[idx], dec[idx], Sloc[idx], S[idx]

    catalog_struct = {}
    catalog_struct["ra"] = ra
    catalog_struct["dec"] = dec
    catalog_struct["Sloc"] = Sloc
    catalog_struct["S"] = S

    if params["doPlots"]:
        catalogfile = os.path.join(params["outputDir"],'catalog.csv')
        fid = open(catalogfile,'w')
        cnt = 0
        for a, b, c, d in zip(ra, dec, Sloc, S):
            fid.write("%d %.5f %.5f %.5e %.5e\n"%(cnt,a,b,c,d))
            cnt = cnt + 1
        fid.close()

    return map_struct, catalog_struct





import os, sys
import copy
import numpy as np
import healpy as hp

from astropy.time import Time

import gwemopt.utils
import gwemopt.rankedTilesGenerator
import gwemopt.scheduler

def combine_coverage_structs(coverage_structs):

    coverage_struct_combined = {}
    coverage_struct_combined["data"] = np.empty((0,8))
    coverage_struct_combined["filters"] = np.empty((0,1))
    coverage_struct_combined["ipix"] = []
    coverage_struct_combined["patch"] = []
    coverage_struct_combined["FOV"] = np.empty((0,1))
    coverage_struct_combined["area"] = np.empty((0,1))
    for coverage_struct in coverage_structs:
        coverage_struct_combined["data"] = np.append(coverage_struct_combined["data"],coverage_struct["data"],axis=0)
        coverage_struct_combined["filters"] = np.append(coverage_struct_combined["filters"],coverage_struct["filters"])
        coverage_struct_combined["ipix"] = coverage_struct_combined["ipix"] + coverage_struct["ipix"]
        coverage_struct_combined["patch"] = coverage_struct_combined["patch"] + coverage_struct["patch"]
        coverage_struct_combined["FOV"] = np.append(coverage_struct_combined["FOV"],coverage_struct["FOV"])
        coverage_struct_combined["area"] = np.append(coverage_struct_combined["area"],coverage_struct["area"])

    return coverage_struct_combined

def read_coverage(params, telescope, filename):

    nside = params["nside"]
    config_struct = params["config"][telescope]

    lines = [line.rstrip('\n') for line in open(filename)]
    lines = lines[1:]
    lines = filter(None,lines)

    coverage_struct = {}
    coverage_struct["data"] = np.empty((0,8))
    coverage_struct["filters"] = []
    coverage_struct["ipix"] = []
    coverage_struct["patch"] = []
    coverage_struct["area"] = []

    for line in lines:
        lineSplit = line.split(",")
        ra = float(lineSplit[2])
        dec = float(lineSplit[3])
        mjd = float(lineSplit[4])
        filt = lineSplit[6]
        mag = float(lineSplit[7])

        coverage_struct["data"] = np.append(coverage_struct["data"],np.array([[ra,dec,mjd,mag,config_struct["exposuretime"],-1,-1,-1]]),axis=0)
        coverage_struct["filters"].append(filt)

        if telescope == "ATLAS":
            alpha=0.2
            color='#6c71c4'
        elif telescope == "PS1":
            alpha=0.1
            color='#859900'
        else:
            alpha=0.2
            color='#6c71c4'

        if config_struct["FOV_coverage_type"] == "square":
            ipix, radecs, patch, area = gwemopt.utils.getSquarePixels(ra, dec, config_struct["FOV_coverage"], nside, alpha=alpha, color=color)
        elif config_struct["FOV_coverage_type"] == "circle":
            ipix, radecs, patch, area = gwemopt.utils.getCirclePixels(ra, dec, config_struct["FOV_coverage"], nside, alpha=alpha, color=color)

        coverage_struct["patch"].append(patch)
        coverage_struct["ipix"].append(ipix)
        coverage_struct["area"].append(area)

    coverage_struct["filters"] = np.array(coverage_struct["filters"])
    coverage_struct["area"] = np.array(coverage_struct["area"])
    coverage_struct["FOV"] = config_struct["FOV_coverage"]*np.ones((len(coverage_struct["filters"]),))

    return coverage_struct

def read_coverage_files(params):

    coverage_structs = []
    for telescope, coverageFile in zip(params["telescopes"],params["coverageFiles"]):
        coverage_struct = read_coverage(params,telescope,coverageFile)
        coverage_structs.append(coverage_struct)

    return combine_coverage_structs(coverage_structs)

def waw(params, map_struct, tile_structs): 

    nside = params["nside"]

    t = np.arange(0,7,1/24.0)
    #t = np.arange(0,7,1.0)
    cr90 = map_struct["cumprob"] < 0.9
    detmaps = gwemopt.waw.detectability_maps(params, t, map_struct, verbose=True, limit_to_region=cr90, nside=nside)

    coverage_structs = []
    for telescope in params["telescopes"]: 
        tile_struct = tile_structs[telescope]
        config_struct = params["config"][telescope]
        T_int = config_struct["exposuretime"]
        ranked_tile_probs = gwemopt.tiles.compute_tiles_map(tile_struct, map_struct["prob"], func='np.sum(x)')
        strategy_struct = gwemopt.waw.construct_followup_strategy_tiles(map_struct["prob"],detmaps,t,tile_struct,T_int,params["Tobs"])
        if strategy_struct is None:
            print("Change distance scale...")
            exit(0)
        print(np.sum(strategy_struct))
        strategy_struct = strategy_struct*86400.0
        keys = tile_struct.keys()
        for key, prob, exposureTime in zip(keys, ranked_tile_probs, strategy_struct):
            tile_struct[key]["prob"] = prob
            tile_struct[key]["exposureTime"] = exposureTime
            tile_struct[key]["nexposures"] = int(np.floor(exposureTime/config_struct["exposuretime"]))
        coverage_struct = gwemopt.scheduler.scheduler(params, config_struct, tile_struct)
        coverage_structs.append(coverage_struct)

    if params["doPlots"]:
        gwemopt.plotting.waw(params,detmaps,t,strategy_struct)

    return combine_coverage_structs(coverage_structs)

def powerlaw(params, map_struct, tile_structs):

    coverage_structs = []
    n_scope = 0
    full_prob_map = map_struct["prob"]

    for telescope in params["telescopes"]:

        if params["doSplit"]:
            if "observability" in map_struct:
                map_struct["observability"][telescope]["prob"] = map_struct["groups"][n_scope]
            else:
                map_struct["prob"] = map_struct["groups"][n_scope]
            if n_scope < len(map_struct["groups"]) - 1:
                n_scope += 1
            else:
                n_scope = 0

        config_struct = params["config"][telescope]
        tile_struct = tile_structs[telescope]

        if params["doAlternatingFilters"]:
            filters, exposuretimes = params["filters"], params["exposuretimes"]
            tile_struct_hold = copy.copy(tile_struct)
            coverage_structs_hold = []
            cnt = 0
            for filt, exposuretime in zip(filters,exposuretimes):
                params["filters"] = [filt]
                params["exposuretimes"] = [exposuretime]
                
                if cnt > 0:
                   if len(coverage_struct_hold["exposureused"]) > 0:
                       maxidx = (np.max(coverage_struct_hold["exposureused"])+1).astype(int)
                       if maxidx > len(config_struct["exposurelist"]):
                           exposurelistnew = config_struct["exposurelist"][maxidx:]
                           config_struct["exposurelist"] = exposurelistnew

                tile_struct_hold = gwemopt.tiles.powerlaw_tiles_struct(params, config_struct, telescope, map_struct, tile_struct_hold)      
                coverage_struct_hold = gwemopt.scheduler.scheduler(params, config_struct, tile_struct_hold)
                coverage_structs_hold.append(coverage_struct_hold)
                cnt = cnt + 1
            coverage_struct = combine_coverage_structs(coverage_structs_hold)
        else:
            tile_struct = gwemopt.tiles.powerlaw_tiles_struct(params, config_struct, telescope, map_struct, tile_struct)      
            coverage_struct = gwemopt.scheduler.scheduler(params, config_struct, tile_struct)
        coverage_structs.append(coverage_struct)

    map_struct["prob"] = full_prob_map
    return combine_coverage_structs(coverage_structs)

def pem(params, map_struct, tile_structs):

    coverage_structs = []
    for telescope in params["telescopes"]:
        config_struct = params["config"][telescope]
        tile_struct = tile_structs[telescope]

        if params["doAlternatingFilters"]:
            filters, exposuretimes = params["filters"], params["exposuretimes"]
            tile_struct_hold = copy.copy(tile_struct)
            coverage_structs_hold = []
            for filt, exposuretime in zip(filters,exposuretimes):
                params["filters"] = [filt]
                params["exposuretimes"] = [exposuretime]
                tile_struct_hold = gwemopt.tiles.pem_tiles_struct(params, config_struct, telescope, map_struct, tile_struct_hold)
                coverage_struct_hold = gwemopt.scheduler.scheduler(params, config_struct, tile_struct_hold)
                coverage_structs_hold.append(coverage_struct_hold)
            coverage_struct = combine_coverage_structs(coverage_structs_hold)
        else:
            tile_struct = gwemopt.tiles.pem_tiles_struct(params, config_struct, telescope, map_struct, tile_struct)
            coverage_struct = gwemopt.scheduler.scheduler(params, config_struct, tile_struct)
        coverage_structs.append(coverage_struct)

    return combine_coverage_structs(coverage_structs)

def timeallocation(params, map_struct, tile_structs):

    if params["timeallocationType"] == "powerlaw":
        print("Generating powerlaw schedule...")
        coverage_struct = gwemopt.coverage.powerlaw(params, map_struct, tile_structs)
    elif params["timeallocationType"] == "waw":
        if params["do3D"]:
            print("Generating WAW schedule...")
            coverage_struct = gwemopt.coverage.waw(params, map_struct, tile_structs)
        else:
            print("Need to enable --do3D for waw")
            exit(0)
    elif params["timeallocationType"] == "pem":
        print("Generating PEM schedule...")
        coverage_struct = gwemopt.coverage.pem(params, map_struct, tile_structs)

    return coverage_struct 



"""
Use extended Hungarian algorithm to minimize/maximize the cost/probability of the scheduling matrix
"""

__author__ = "Javed Rana <javed@iucaa.in>"


### Imports
import numpy as np
import multiprocessing as mp

class Hungarian:
    '''Use Hungarian algorithm to minimize the cost'''

    def __init__(self, matrix, profit_matrix=False):
        '''Input the cost matrix.
        If the profit_matrix is true then make it the cost matrix
        '''
        self.assigned_points = []
        newmatrix = np.array(matrix)
        ### Padding of zeros if the matrix is not balanced.
        matrix_shape = newmatrix.shape
        max_size = max(matrix_shape)
        if max_size > 100*matrix_shape[1]:
            max_size = 100*matrix_shape[1]
            newmatrix = newmatrix[:max_size]
        matrix_shape = newmatrix.shape
        pad_row = max_size - matrix_shape[0]
        pad_column = max_size - matrix_shape[1]
        newmatrix = np.pad(newmatrix, ((0,pad_row),(0,pad_column)), 'constant', constant_values=(0.))
        
        if profit_matrix:
            max_element = newmatrix.max()
            matrix_shape = newmatrix.shape
            equal_matrix = np.ones(matrix_shape, dtype=int) * max_element
            newmatrix = equal_matrix - newmatrix
        self.cost_matrix = newmatrix.copy()
        self.shape = matrix_shape


    ### Phase 1
    ########################################################################
    ### Step 1
    def row_column_subtraction(self):
        """Subtract the minimum element of each row from that row"""
        if self.cost_matrix is not None:
            # Minimum element subtraction from each row 
            for index, row in enumerate(self.cost_matrix):
                self.cost_matrix[index] -= row.min()
            # Minimum element subctration from each column
            for index, column in enumerate(self.cost_matrix.T):
                self.cost_matrix.T[index] -= column.min()

    ### Phase 2
    ### Step 2
    def calculate(self):
        """Return the optimal solution"""
        ### Step 1
        self.row_column_subtraction()

        ### Step 2
        itr = 0
        optimal_row = []
        while len(optimal_row) < len(self.cost_matrix):
            ### Initial assigning of the jobs
            zerolines = ZeroLines(self.cost_matrix)
            zerolines.calculate()
            init_assignment = zerolines.assigned_points

            check_matrix = self.cost_matrix.copy()
            ### Mark all the zeros with minimum lines
            markedRC = markRowColumn(self.cost_matrix)
            markedRC.calculate(init_assignment)
            optimal_line = markedRC.optimal_line
            ### Check whether any farther improvement is possible
            if (check_matrix == self.cost_matrix).all():
                break
            if optimal_line == len(self.cost_matrix):
                break
        self.assigned_points = zerolines.assigned_points

### Step 2
### Find the minimum line to cover the all zeros
class ZeroLines:
    '''Cover the zeros assigning in the the jobs in rows and collumns
    If a row has one zero, assign that row.
    If a row has more than one zero skip that row.
    After all row assignment, assign the columns with one zero.
    If a column has more than one zeroskip that column.
    Repeat the row and column assignments untill all the zeros are eighter assigned or crossed
    '''

    def __init__(self, cost_matrix):
        """Input cost matrix and save the position of the optimal cost"""
        self.cost_matrix = cost_matrix
        self.optimal_row = []
        self.optimal_column = []
        self.zero_mask = [] 
        self.position_matrix = []
        self.assigned_points = []

    def rowassign(self, row_mask, matrix_column):
        '''Find the zeros in the row and return the column index of the first zero'''
        return matrix_column[row_mask][0]

    def newmatrix(self, delete_index, axis=0):
        '''Delete the assigned column'''
        self.zero_mask = np.delete(self.zero_mask, delete_index, axis=axis)
        self.position_matrix = np.delete(self.position_matrix, delete_index, axis=axis)

    def deleteindex(self, row_mask):
        '''Return the index to delete'''
        ar = np.arange(len(row_mask))
        return ar[row_mask][0]

    def rowScanning(self):
        '''If a row has one zero, assign that row.
        If a row has more than one zero skip that row.
        '''
        rowlen = len(self.zero_mask)
        for row_index in range(rowlen):
            row_mask = self.zero_mask[row_index]
            if row_mask.sum()==1:
                cindex_assignrow = self.rowassign(row_mask, self.position_matrix[row_index])
                self.assigned_points.append(cindex_assignrow)
                delete_index = self.deleteindex(row_mask)
                ### The changed matrix
                self.newmatrix(delete_index, axis=1)

    def columnScanning(self):
        '''If a column has one zero, assign that column.
        If a column has more than one zeroskip that column.
        '''
        columnlen = len(self.zero_mask.T)
        for column_index in range(columnlen):
            column_mask = self.zero_mask[:,column_index]
            if column_mask.sum()==1:
                rindex_assigncolumn = self.rowassign(column_mask, self.position_matrix[:,column_index])
                self.assigned_points.append(rindex_assigncolumn)
                delete_index = self.deleteindex(column_mask)
                ### The changed matrix
                self.newmatrix(delete_index, axis=0)

    def coverExtraZeros(self):
        '''If extra zeros are left after row and column scanning,
        cover them by arbitrary assignment.
        '''
        rowlen = len(self.zero_mask)
        for row_index in range(rowlen):
            row_mask = self.zero_mask[row_index]
            if row_mask.any():
                cindex_assignrow = self.rowassign(row_mask, self.position_matrix[row_index])
                self.assigned_points.append(cindex_assignrow)
                delete_index = self.deleteindex(row_mask)
                ### The changed matrix
                self.newmatrix(delete_index, axis=1)

    def calculate(self):
        """Find the index of the rows and columns having zeros
        and the indexes of the zreos"""
        rowlen = len(self.cost_matrix)
        columnlen = len(self.cost_matrix.T)
        row_indexes = np.arange(rowlen)
        column_indexes = np.arange(columnlen)
        
        self.position_matrix = np.zeros(shape=(rowlen,columnlen,2), dtype=int)
        for ri in range(rowlen):
            for ci in range(columnlen):
                self.position_matrix[ri,ci] = [ri,ci]
        self.zero_mask = self.cost_matrix == 0.
        copy_zero_mask = self.zero_mask.copy()
        itr = 0
        while self.zero_mask.any():
            self.rowScanning()
            if self.zero_mask is not None:
                self.columnScanning()
            itr += 1
            if itr>100:
                self.coverExtraZeros()


### Mark the unassigned rows and 
class markRowColumn:
    '''Mark all rows having no assignments.
    Mark all (unmarked) columns having zeros in newly marked rows.
    Mark all rows having assignments in newly marked columns.
    Repeat for all non-assigned rows.
    Draw lines through all marked columns and unmarked rows.
    '''

    def __init__(self, cost_matrix):
        '''Input cost matrix and the assigned row/columns'''
        self.cost_matrix = cost_matrix
        self.marked_rows = []
        self.unmarked_rows = []
        self.marked_columns = []
        self.unmarked_columns = []
        self.optimal_line = 0

    def zeroUnassignedRow(self, row):
        '''Find the zero in the unmarked rows'''
        row_mask = row == 0.
        if row_mask.any():
            ar = np.arange(len(row))
            return ar[row_mask]
        else: return []

    def uncoveredMatrix(self):
        '''Delete the all marked columns and unmarked rows'''
        f_matrix = np.delete(self.cost_matrix, self.marked_columns, axis=1)
        f_matrix = np.delete(f_matrix, self.unmarked_rows, axis=0)
        return f_matrix

    def __mark_row_column(self, unassigned_rows, assigned_columns, assigned_rows):
        '''Mark all (unmarked) columns having zeros in newly marked row.
        Mark all rows having assignments in newly marked columns.
        '''
        def _marking():
            '''First marking of the rows and columns from the unassigned 
            rows
            '''
            pool = mp.Pool(mp.cpu_count())
            cindex = pool.map(self.zeroUnassignedRow, [self.cost_matrix[ri] for ri in self.marked_rows])
            pool.close()
            for i in range(len(cindex)):
                # cindex = self.zeroUnassignedRow(self.cost_matrix[ri])
                for ci in cindex[i]:
                    self.marked_columns.append(ci)
            for ci in self.marked_columns:
                if ci in assigned_columns:
                    index = np.argmin(abs(assigned_columns-ci))
                    self.marked_rows.append(assigned_rows[index])
            self.marked_rows = list(set(self.marked_rows))
            self.marked_columns = list(set(self.marked_columns))
            self.unmarked_rows = np.delete(np.arange(len(self.cost_matrix)), self.marked_rows)

        r_matrix = self.uncoveredMatrix()
        r_matrix_mask = r_matrix == 0
        itr = 0
        while r_matrix_mask.any():
            _marking()
            r_matrix = self.uncoveredMatrix()
            r_matrix_mask = r_matrix == 0

    def minElementSub(self, f_matrix, intersect_points):
        '''Subtract the minimum element from all uncovered element
        Add the minimum element to all elements covered by two lines'''
        min_element = np.min(f_matrix)
        ### Subtract the minimum element from all uncovered element
        for ci in self.unmarked_columns:
            for ri in self.marked_rows:
                self.cost_matrix[ri,ci] -= min_element
        ### Add the minimum element to all elements covered by two lines
        for point in intersect_points:
            self.cost_matrix[point[0],point[1]] += min_element 

    def calculate(self, assigned_points):
        '''Calculate marked'''
        assigned_rows = (np.array(assigned_points).T)[0]
        assigned_columns = (np.array(assigned_points).T)[1]
        self.marked_rows = [ri for ri in range(len(self.cost_matrix)) if ri not in assigned_rows]
        unassigned_rows = self.cost_matrix[self.marked_rows]

        self.__mark_row_column(unassigned_rows, assigned_columns, assigned_rows)
        self.marked_columns = np.unique(np.array(self.marked_columns))
        self.marked_rows = np.unique(np.array(self.marked_rows))
        self.unmarked_rows = np.delete(np.arange(len(self.cost_matrix)), self.marked_rows)
        self.unmarked_columns = np.delete(np.arange(len(self.cost_matrix)), self.marked_columns)
        intersect_points = []
        for ri in self.unmarked_rows:
            for ci in self.marked_columns:
                intersect_points.append([ri,ci])

        f_matrix = self.uncoveredMatrix()
        if len(f_matrix):
            self.minElementSub(f_matrix, intersect_points)
        self.optimal_line = len(self.unmarked_rows) + len(self.marked_columns)




# -*- coding: utf-8 -*-
# Copyright (C) Michael Coughlin and Christopher Stubbs(2015)
#
# skybrightness is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gwemopt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with skybrightness.  If not, see <http://www.gnu.org/licenses/>.

"""This module provides example methods to calculate the optimal likelihood and distance scalings for optimizing telescope time allocations.
"""

from __future__ import division

import os, sys, pickle, math, optparse, glob, time, subprocess
from datetime import date, datetime
import numpy as np

seed = 1
np.random.seed(seed=seed)

import scipy.ndimage
import astropy

import healpy as hp

import scipy.io
import h5py

import matplotlib
#matplotlib.rc('text', usetex=True)
matplotlib.use('Agg')
matplotlib.rcParams.update({'font.size': 16})
matplotlib.rcParams['contour.negative_linestyle'] = 'solid'
import matplotlib.pyplot as plt
from matplotlib import cm as cmx
from matplotlib import colors
from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection
from matplotlib.mlab import griddata

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

import pymultinest

__author__ = "Michael Coughlin <michael.coughlin@ligo.org>"
__version__ = 1.0
__date__    = "9/22/2013"

def parse_commandline():
    """@Parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-o","--outputDir",help="Output location",default = '/home/mcoughlin/Skymaps/optimization/plots')
    parser.add_option("-d","--dataDir",help="Data location",default  = '/home/mcoughlin/Skymaps/going-the-distance_data/2015/compare/')

    parser.add_option("-t", "--telescope", help="Telescope.",
                      default ="PS1")

    parser.add_option("--minabsm", help="Min absolute magnitude.",default=-16,type=float)
    parser.add_option("--maxabsm", help="Max absolute magnitude.",default=-10,type=float)    
    parser.add_option("--minfade", help="Min time above detection threshold [hours].",default=1,type=float)
    parser.add_option("--maxfade", help="Max time above detection threshold [hours].",default=10,type=float)

    parser.add_option("-e", "--event", help="Event.",default = None)
    parser.add_option("-n", "--nsignals", help="Number of simulated signals.", type=int, default = 10000)

    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Run verbosely. (Default: False)")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running pylal_seismon_run..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts


def get_post_file(basedir):
    filenames = glob.glob(os.path.join(basedir,'2-post*'))
    if len(filenames)>0:
        filename = filenames[0]
    else:
        filename = []
    return filename

def myprior(cube, ndim, nparams):
        cube[0] = cube[0]*8.0 - 4.0
        cube[1] = cube[1]*1.0
        cube[2] = cube[2]*8.0 - 4.0

def myprior_multi(cube, ndim, nparams):
        cube[0] = cube[0]*8.0 - 4.0
        cube[1] = cube[1]*1.0
        cube[2] = cube[2]*8.0 - 4.0
        cube[3] = cube[3]*8.0 - 4.0
        cube[4] = cube[4]*1.0
        cube[5] = cube[5]*8.0 - 4.0

def myloglike(cube, ndim, nparams):
        n = cube[0]
        cl = cube[1]
        r = cube[2]

        image_array = numimages(prob_data,distmu_data,data_out,Telescope_T,Telescope_m,n,cl,r)

        image_array_sorted = np.sort(image_array)

        tmax = data_out["fade"]*3600
        image_detected = float(len(np.where(image_array < tmax)[0]))/float(len(image_array))

        index = np.floor(len(image_array)*0.5)
        image_T = image_array_sorted[index]/3600.0

        #prob = 1/image_detected
        prob = image_detected
        #prob = -np.log10(1-image_detected)
        #prob = -np.log10(image_T)

        #if np.isfinite(prob):
        #    print n, cl, r, prob
        return prob

def myloglike_multi(cube, ndim, nparams):
        n_PS1 = cube[0]
        cl_PS1 = cube[1]
        r_PS1 = cube[2]
        n_ATLAS = cube[3]
        cl_ATLAS = cube[4]
        r_ATLAS = cube[5]

        image_array = numimages_combine(prob_data,distmu_data,data_out,n_PS1,cl_PS1,r_PS1,n_ATLAS,cl_ATLAS,r_ATLAS)

        image_array_sorted = np.sort(image_array)

        tmax = data_out["fade"]*3600
        image_detected = float(len(np.where(image_array < tmax)[0]))/float(len(image_array))

        index = np.floor(len(image_array)*0.5)
        image_T = image_array_sorted[index]/3600.0

        prob = image_detected

        #if np.isfinite(prob):
        #    print n_PS1, cl_PS1, r_PS1, n_ATLAS, cl_ATLAS, r_ATLAS, prob
        return prob

def probpower(prob_data,distmu_data,n,cl,r):

    prob_data_sorted = np.sort(prob_data)[::-1]
    prob_data_indexes = np.argsort(prob_data)[::-1]
    prob_data_cumsum = np.cumsum(prob_data_sorted)
    index = np.argmin(np.abs(prob_data_cumsum - cl)) + 1

    prob_data_scaled = prob_data.copy()
    prob_data_scaled[prob_data_indexes[index:]] = 0.0
    prob_data_scaled = prob_data_scaled**n
    distmu_data_scaled = (distmu_data/np.nanmax(distmu_data))**r
    prob_data_scaled = prob_data_scaled * distmu_data_scaled
    prob_data_scaled[np.isnan(prob_data_scaled)] = 0.0
    prob_data_scaled = prob_data_scaled / np.nansum(prob_data_scaled)

    return prob_data_scaled 

def numimages(prob_data,distmu_data,data_out,Telescope_T,Telescope_m,n,cl,r):

    phi_true = data_out["ra"]
    theta_true = 0.5*np.pi - data_out["dec"]
    index_true = hp.ang2pix(nside, theta_true, phi_true)
    appm = data_out["absm"] + 5*(np.log10(data_out["dist"]*1e6) - 1)

    Telescope_nimages = 2.5**(appm - Telescope_m)
    Telescope_T_scaled = Telescope_T*Telescope_nimages

    prob_data_scaled = probpower(prob_data,distmu_data,n,cl,r)
    prob_data_true = prob_data_scaled[index_true]

    image_array = Telescope_T_scaled/prob_data_true

    return image_array

def numimages_combine(prob_data,distmu_data,data_out,n_PS1,cl_PS1,r_PS1,n_ATLAS,cl_ATLAS,r_ATLAS):
    image_array_PS1 = numimages(prob_data,distmu_data,data_out,PS1_Telescope_T,PS1_Telescope_m,n_PS1,cl_PS1,r_PS1)
    image_array_ATLAS = numimages(prob_data,distmu_data,data_out,ATLAS_Telescope_T,ATLAS_Telescope_m,n_ATLAS,cl_ATLAS,r_ATLAS)
    image_array = np.min(np.vstack((image_array_PS1,image_array_ATLAS)),axis=0)

    return image_array

def numknown(prob_data,data_out,Telescope_T,Telescope_m):

    phi_true = data_out["ra"]
    theta_true = 0.5*np.pi - data_out["dec"]
    index_true = hp.ang2pix(nside, theta_true, phi_true)
    appm = data_out["absm"] + 5*(np.log10(data_out["dist"]*1e6) - 1)

    Telescope_nimages = 2.5**(appm - Telescope_m)
    Telescope_T_scaled = Telescope_T*Telescope_nimages

    return Telescope_T_scaled

def hist_results(samples):

    bins = np.linspace(np.min(samples),np.max(samples),11)
    hist1, bin_edges = np.histogram(samples, bins=bins)
    hist1 = hist1 / float(np.sum(hist1))
    bins = (bins[1:] + bins[:-1])/2.0

    return bins, hist1

def healpix(filename,plotDir,data_out):

    bayestar = True
    if bayestar:
        healpix_data = hp.read_map(filename, field=(0,1,2,3))

        distmu_data = healpix_data[1]
        diststd_data = healpix_data[2]
        prob_data = healpix_data[0]
        norm_data = healpix_data[3]

        plotName = os.path.join(plotDir,'mollview.png')
        vmin = np.min(prob_data)
        vmax = np.max(prob_data)
        hp.mollview(prob_data, min=vmin,max=vmax,title="",unit='Likelihood')
        hp.graticule()
        plt.show()
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'mollview.eps')
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'mollview.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')

        distmu_data[distmu_data<0] = np.inf
        distmu_data_cut = distmu_data[~np.isinf(distmu_data)] 

        plotName = os.path.join(plotDir,'mollview_dist.png')
        vmin = np.min(distmu_data_cut)
        vmax = np.max(distmu_data_cut)

        hp.mollview(distmu_data, min=vmin,max=vmax,title="",unit='Distance [Mpc]')
        hp.graticule()
        plt.show()
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'mollview_dist.eps')
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'mollview_dist.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')

        diststd_data[diststd_data<0] = np.inf
        diststd_data_cut = diststd_data[~np.isinf(diststd_data)]

        plotName = os.path.join(plotDir,'mollview_dist_std.png')
        vmin = np.min(diststd_data_cut)
        vmax = np.max(diststd_data_cut)

        hp.mollview(diststd_data, min=vmin,max=vmax,title="",unit='Distance [Mpc]')
        hp.graticule()
        plt.show()
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'mollview_dist_std.eps')
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'mollview_dist_std.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')

        distmu_data = hp.ud_grade(distmu_data,nside)
        diststd_data = hp.ud_grade(diststd_data,nside)
        norm_data = hp.ud_grade(norm_data,nside)
    else:
        prob_data = hp.read_map(filename, field=0)
        prob_data = prob_data / np.sum(prob_data)    
    prob_data = hp.ud_grade(prob_data,nside)
    prob_data = prob_data / np.sum(prob_data)

    indexes = np.where(prob_data < np.max(prob_data)*0.01)[0]
    prob_data_nan = prob_data.copy()
    prob_data_nan[indexes] = np.nan
    vmin = np.nanmin(prob_data_nan)
    vmax = np.nanmax(prob_data_nan)

    plotName = os.path.join(plotDir,'mollview_small.png')
    hp.mollview(prob_data_nan, min=vmin,max=vmax,title="",unit='Likelihood')
    hp.graticule()
    plt.show()
    plt.savefig(plotName,dpi=200)
    plotName = os.path.join(plotDir,'mollview_small.eps')
    plt.savefig(plotName,dpi=200)
    plotName = os.path.join(plotDir,'mollview_small.pdf')
    plt.savefig(plotName,dpi=200)
    plt.close('all')
   
    npix = hp.nside2npix(nside)
    theta, phi = hp.pix2ang(nside, np.arange(npix))
    ra = np.rad2deg(phi)
    dec = np.rad2deg(0.5*np.pi - theta)
    absm = np.random.rand(len(data_out["ra"]),)*(maxabsm-minabsm) + minabsm
    data_out["absm"] = absm
    fade = np.random.rand(len(data_out["ra"]),)*(maxfade-minfade) + minfade
    data_out["fade"] = fade

    # Run MC
    print "Ra: %.5f %.5f"%(np.min(data_out["ra"]),np.max(data_out["ra"]))
    print "Declination: %.5f %.5f"%(np.min(data_out["dec"]),np.max(data_out["dec"]))
    print "Distance: %.5f %.5f"%(np.min(data_out["dist"]),np.max(data_out["dist"]))
    print "Absolute Magnitude: %.5f %.5f"%(np.min(data_out["absm"]),np.max(data_out["absm"]))  

    if telescope == "combined":
        # number of dimensions our problem has
        parameters = ["n_PS1","cl_PS1","r_PS1","n_ATLAS","cl_ATLAS","r_ATLAS"]
    else:
        # number of dimensions our problem has
        parameters = ["n","cl","r"]
    n_params = len(parameters)

    global prob_data, distmu_data

    if telescope == "combined":
        pymultinest.run(myloglike_multi, myprior_multi, n_params, importance_nested_sampling = False, resume = True, verbose = True, sampling_efficiency = 'parameter', n_live_points = 1000, outputfiles_basename='%s/2-'%plotDir, evidence_tolerance = 0.001, multimodal = False)
    else:
        pymultinest.run(myloglike, myprior, n_params, importance_nested_sampling = False, resume = True, verbose = True, sampling_efficiency = 'parameter', n_live_points = 1000, outputfiles_basename='%s/2-'%plotDir, evidence_tolerance = 0.001, multimodal = False)

    # lets analyse the results
    a = pymultinest.Analyzer(n_params = n_params, outputfiles_basename='%s/2-'%plotDir)
    s = a.get_stats()

    import json
    # store name of parameters, always useful
    with open('%sparams.json' % a.outputfiles_basename, 'w') as f:
                json.dump(parameters, f, indent=2)
    # store derived stats
    with open('%sstats.json' % a.outputfiles_basename, mode='w') as f:
                json.dump(s, f, indent=2)
    print()
    print("-" * 30, 'ANALYSIS', "-" * 30)
    print("Global Evidence:\n\t%.15e +- %.15e" % ( s['nested sampling global log-evidence'], s['nested sampling global log-evidence error'] ))

    #multifile= os.path.join(plotDir,'2-.txt')
    multifile = get_post_file(plotDir)
    data = np.loadtxt(multifile)

    #loglikelihood = -(1/2.0)*data[:,1]
    #idx = np.argmax(loglikelihood)

    if telescope == "combined":

        n_PS1 = data[:,0]
        cl_PS1 = data[:,1]
        r_PS1 = data[:,2]
        n_ATLAS = data[:,3]
        cl_ATLAS = data[:,4]
        r_ATLAS = data[:,5]
        loglikelihood = data[:,6]
        idx = np.argmax(loglikelihood)
    
        n_PS1_best = data[idx,0]
        cl_PS1_best = data[idx,1]
        r_PS1_best = data[idx,2]
        n_ATLAS_best = data[idx,3]
        cl_ATLAS_best = data[idx,4]
        r_ATLAS_best = data[idx,5]
    
        image_optimal_array = numimages_combine(prob_data,distmu_data,data_out,n_PS1_best,cl_PS1_best,r_PS1_best,n_ATLAS_best,cl_ATLAS_best,r_ATLAS_best)
        image_optimal_array_sorted = np.sort(image_optimal_array)
    
        image_nominal_array = numimages_combine(prob_data,distmu_data,data_out,0.0,0.99,0.0,0.0,0.99,0.0)
        image_nominal_array_sorted = np.sort(image_nominal_array)
    
        image_known_array = numknown(prob_data,data_out,PS1_Telescope_T,PS1_Telescope_m)
        image_known_array_sorted = np.sort(image_known_array)
    
        filename = os.path.join(plotDir,'samples.dat')
        fid = open(filename,'w+')
        for i, j, k, l, m, o in zip(n_PS1,cl_PS1,r_PS1,n_ATLAS,cl_ATLAS,r_ATLAS):
            fid.write('%.5f %.5f %.5f %.5f %.5f %.5f\n'%(i,j,k,l,m,o))
        fid.close()
    
        filename = os.path.join(plotDir,'best.dat')
        fid = open(filename,'w')
        fid.write('%.5f %.5f %.5f %.5f %.5f %.5f\n'%(n_PS1_best,cl_PS1_best,r_PS1_best,n_ATLAS_best,cl_ATLAS_best,r_ATLAS_best))
        fid.close()
    
        plt.figure(figsize=(12,10))
        bins1, hist1 = hist_results(n_PS1)
        bins2, hist2 = hist_results(n_ATLAS)
        #plt.plot(bins1, np.cumsum(hist1),'k',label='PS1')
        #plt.plot(bins2, np.cumsum(hist2),'k--',label='ATLAS')
        plt.plot(bins1, hist1,'k',label='PS1')
        plt.plot(bins2, hist2,'k--',label='ATLAS')
        plt.legend(loc='best')
        #plt.xlim([10,1e6])
        #plt.ylim([0,1.0])
        plt.xlabel('Likelihood Powerlaw Index')
        plt.ylabel('Probability Density Function')
        plt.show()
        plotName = os.path.join(plotDir,'n.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')
    
        plt.figure(figsize=(12,10))
        bins1, hist1 = hist_results(cl_PS1)
        bins2, hist2 = hist_results(cl_ATLAS)
        #plt.plot(bins1, np.cumsum(hist1),'k',label='PS1')
        #plt.plot(bins2, np.cumsum(hist2),'k--',label='ATLAS')
        plt.plot(bins1, hist1,'k',label='PS1')
        plt.plot(bins2, hist2,'k--',label='ATLAS')
        plt.legend(loc='best')
        #plt.xlim([10,1e6])
        #plt.ylim([0,1.0])
        plt.xlabel('Likelihood Confidence Level')
        plt.ylabel('Probability Density Function')
        plt.show()
        plotName = os.path.join(plotDir,'cl.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')
    
        plt.figure(figsize=(12,10))
        bins1, hist1 = hist_results(r_PS1)
        bins2, hist2 = hist_results(r_ATLAS)
        #plt.plot(bins1, np.cumsum(hist1),'k',label='PS1')
        #plt.plot(bins2, np.cumsum(hist2),'k--',label='ATLAS')
        plt.plot(bins1, hist1,'k',label='PS1')
        plt.plot(bins2, hist2,'k--',label='ATLAS')
        plt.legend(loc='best')
        #plt.xlim([10,1e6])
        #plt.ylim([0,1.0])
        plt.xlabel('Distance Powerlaw Index')
        plt.ylabel('Probability Density Function')
        plt.show()
        plotName = os.path.join(plotDir,'r.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')

    else:
        n = data[:,0]
        cl = data[:,1]
        r = data[:,2]
        loglikelihood = data[:,3]
        idx = np.argmax(loglikelihood)

        n_best = data[idx,0]
        cl_best = data[idx,1]
        r_best = data[idx,2]

        image_optimal_array = numimages(prob_data,distmu_data,data_out,Telescope_T,Telescope_m,n_best,cl_best,r_best)
        image_optimal_array_sorted = np.sort(image_optimal_array)

        image_nominal_array = numimages(prob_data,distmu_data,data_out,Telescope_T,Telescope_m,0.0,0.99,0.0)
        image_nominal_array_sorted = np.sort(image_nominal_array)

        image_known_array = numknown(prob_data,data_out,Telescope_T,Telescope_m)
        image_known_array_sorted = np.sort(image_known_array)

        filename = os.path.join(plotDir,'samples.dat')
        fid = open(filename,'w+')
        for i, j, k in zip(n,cl,r):
            fid.write('%.5f %.5f %.5f\n'%(i,j,k))
        fid.close()

        filename = os.path.join(plotDir,'best.dat')
        fid = open(filename,'w')
        fid.write('%.5f %.5f %.5f\n'%(n_best,cl_best,r_best))
        fid.close()

        plt.figure(figsize=(12,10))
        bins1, hist1 = hist_results(n)
        plt.plot(bins1, hist1)
        plt.xlabel('Powerlaw Index')
        plt.ylabel('Probability Density Function')
        plt.show()
        plotName = os.path.join(plotDir,'n.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')
    
        plt.figure(figsize=(12,10))
        bins1, hist1 = hist_results(cl)
        plt.plot(bins1, hist1)
        plt.xlabel('Powerlaw Index')
        plt.ylabel('Probability Density Function')
        plt.show()
        plotName = os.path.join(plotDir,'cl.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')
    
        plt.figure(figsize=(12,10))
        bins1, hist1 = hist_results(r)
        plt.plot(bins1, hist1)
        plt.xlabel('Powerlaw Index')
        plt.ylabel('Probability Density Function')
        plt.show()
        plotName = os.path.join(plotDir,'r.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')

        ns = np.linspace(0.01,2.0,50)
        cls = np.linspace(0.5,1.0,50)

        images_detected = np.zeros((len(ns),len(cls)))
        images_T = np.zeros((len(ns),len(cls)))

        for ii,n in enumerate(ns):
            for jj,cl in enumerate(cls):
                image_array = numimages(prob_data,distmu_data,data_out,Telescope_T,Telescope_m,n,cl,0.0)
                image_array_sorted = np.sort(image_array)

                tmax = tmax = data_out["fade"]*3600
                image_detected = float(len(np.where(image_array < tmax)[0]))/float(len(image_array))

                index = np.floor(len(image_array)*0.5)
                image_T = image_array_sorted[index]

                images_detected[ii,jj] = image_detected
                images_T[ii,jj] = image_T/3600.0

        NS,CLS = np.meshgrid(ns,cls)

        vmin = np.min(images_detected)
        vmax = np.max(images_detected)
        plotName = os.path.join(plotDir,'images_detected.png')
        plt.figure()
        plt.pcolor(NS,CLS,images_detected.T, vmin = vmin, vmax = vmax, cmap = plt.get_cmap('rainbow'))
        cbar = plt.colorbar()
        cbar.ax.set_ylabel('Percentage of detections')
        plt.xlabel('Power Law Index')
        plt.ylabel('Confidence Level')
        plt.xlim([0.01,2.0])
        plt.ylim([0.5,1.0])
        plt.show()
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'images_detected.eps')
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'images_detected.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')

        vmin = 0.0
        vmax = 10.0
        plotName = os.path.join(plotDir,'images_T.png')
        plt.figure()
        plt.pcolor(NS,CLS,images_T.T, vmin = vmin, vmax = vmax, cmap = plt.get_cmap('rainbow'))
        cbar = plt.colorbar()
        cbar.ax.set_ylabel('Median detection time [hours]')
        plt.xlabel('Power Law Index')
        plt.ylabel('Confidence Level')
        plt.xlim([0.01,2.0])
        plt.ylim([0.5,1.0])
        plt.show()
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'images_T.eps')
        plt.savefig(plotName,dpi=200)
        plotName = os.path.join(plotDir,'images_T.pdf')
        plt.savefig(plotName,dpi=200)
        plt.close('all')

    nums = np.arange(0,len(image_nominal_array_sorted))
    nums[:] = 1.0
    nums = nums / np.sum(nums)
    nums_cumsum = np.cumsum(nums)

    plotName = os.path.join(plotDir,'optimization.png')
    plt.figure()
    plt.semilogx(image_nominal_array_sorted,100*nums_cumsum,'k--',label='Naive')
    plt.semilogx(image_optimal_array_sorted,100*nums_cumsum,'r.-',label='Optimal')
    plt.semilogx(image_known_array_sorted,100*nums_cumsum,'c',label='Known')
    plt.legend(loc=2)
    plt.xlim([10,1e6])
    plt.ylim([0,100.0])
    plt.xlabel('Time [s]')
    plt.ylabel('Percentage of imaged counterparts')
    plt.show()
    plt.savefig(plotName,dpi=200)
    plotName = os.path.join(plotDir,'optimization.eps')
    plt.savefig(plotName,dpi=200)
    plotName = os.path.join(plotDir,'optimization.pdf')
    plt.savefig(plotName,dpi=200)
    plt.close('all')

    filename = os.path.join(plotDir,'images.dat')
    fid = open(filename,'w+')
    for i, j, k, l in zip(nums_cumsum,image_nominal_array_sorted,image_optimal_array_sorted,image_known_array_sorted):
        fid.write('%.5e %.5e %.5e %.5e\n'%(i,j,k,l))
    fid.close()

def mkdir(path):
    """@create path (if it does not already exist).

    @param path
        directory path to create
    """

    pathSplit = path.split("/")
    pathAppend = "/"
    for piece in pathSplit:
        if piece == "":
            continue
        pathAppend = os.path.join(pathAppend,piece)
        if not os.path.isdir(pathAppend):
            os.mkdir(pathAppend)

# Parse command line
opts = parse_commandline()
telescope = opts.telescope

outputDir = opts.outputDir
if not os.path.isdir(outputDir):
    os.mkdir(outputDir)
baseplotDir = os.path.join(outputDir,telescope)
if not os.path.isdir(baseplotDir):
    os.mkdir(baseplotDir)

dataDir = opts.dataDir

directories = glob.glob(os.path.join(dataDir,'*'))

minabsm = opts.minabsm
maxabsm = opts.maxabsm

minfade = opts.minfade
maxfade = opts.maxfade

if telescope == "PS1":
    nside = 16
    # PS1
    # 7 sq degrees (3 deg diameter) 
    #for the i-band, which we nominally use 
    #for t=45, 
    # m_limit (5 sigma) = 21.5  (AB mag) 
    # Using nside=32 which is 3.3 square degrees
    # Using nside=16 which is 13.4 square degrees

    Telescope_T = 45.0
    Telescope_m = 21.5
    Telescope_T = Telescope_T* hp.nside2pixarea(nside, degrees=True) / 7.0 
elif telescope == "ATLAS":

    # ATLAS  
    # 29.2 sq degrees
    # T = 30s
    # m_limit = 18.7 (5 sigma) in the cyan band
    # Using nside=16 which is 13.4 square degrees

    nside = 16
    Telescope_T = 30.0
    Telescope_m = 18.7
    Telescope_T = Telescope_T* hp.nside2pixarea(nside, degrees=True) / 29.2
elif telescope == "combined":
    nside = 16

    PS1_Telescope_T = 45.0
    PS1_Telescope_m = 21.5
    PS1_Telescope_T = PS1_Telescope_T*hp.nside2pixarea(nside, degrees=True) / 7.0

    ATLAS_Telescope_T = 30.0
    ATLAS_Telescope_m = 18.7
    ATLAS_Telescope_T = ATLAS_Telescope_T* hp.nside2pixarea(nside, degrees=True) / 29.2

print "Pixel area: %.4f square degrees" % hp.nside2pixarea(nside, degrees=True)

# http://iopscience.iop.org/article/10.3847/0004-637X/825/1/52/pdf
#5*(log10(400*1e6)-1) = 38.0103
# 28 to 22 = -10 to -16

for directory in directories:
    directorySplit = directory.split("/")[-1]
    plotDir = os.path.join(baseplotDir,directorySplit)

    if not opts.event == None:
        if not opts.event == directorySplit: continue
    print plotDir
    print directorySplit

    filename = os.path.join(directory,'bayestar.fits.gz')
 
    if not os.path.isfile(filename):
        continue

    filename = os.path.join(directory,'lalinference_nest/skymap.fits.gz')
    filename_samples = os.path.join(directory,'lalinference_nest/posterior_samples.dat')

    lines = [line.rstrip('\n') for line in open(filename_samples)]
    data = np.loadtxt(filename_samples,skiprows=1)
    line = lines[0]
    params = line.split("\t")
    params = filter(None, params)
    samples = {}

    for ii in xrange(len(params)):
        param = params[ii]
        samples[param] = data[:,ii]

        print "%s: %.5f"%(param,np.median(samples[param]))

    data_out = {}
    data_out["ra"] = []
    data_out["dec"] = []
    data_out["dist"] = []
    for ii in xrange(opts.nsignals):
        idx = np.floor(np.random.rand()*len(samples["ra"]))
        data_out["ra"].append(samples["ra"][idx])
        data_out["dec"].append(samples["dec"][idx])
        data_out["dist"].append(samples["dist"][idx])    
    data_out["ra"] = np.array(data_out["ra"])
    data_out["dec"] = np.array(data_out["dec"])
    data_out["dist"] = np.array(data_out["dist"])

    plotDir = "%s/%s/%d-%d/%.2f-%.2f"%(baseplotDir,directorySplit,minabsm,maxabsm,minfade,maxfade)
    mkdir(plotDir)
    healpix(filename,plotDir,data_out)





import os, sys
import time
import copy

import numpy as np
import healpy as hp

import gwemopt.utils

def hierarchical_tiles_struct(params, config_struct, telescope, map_struct):

    import pymultinest

    Ntiles = params["Ntiles"]

    map_struct_copy = copy.deepcopy(map_struct)

    def myprior(cube, ndim, nparams):
        cube[0] = cube[0]*360.0
        cube[1] = cube[1]*180.0 - 90.0

    def myloglike_square(cube, ndim, nparams):
        ra = cube[0]
        dec = cube[1]

        ipix, radecs, patch, area = gwemopt.utils.getSquarePixels(ra, dec, config_struct["FOV"], nside)
        
        if len(ipix) == 0:
            prob = -np.inf
        else:
            prob = np.sum(map_struct_copy["prob"][ipix])

        if prob == 0:
            prob = -np.inf

        #if np.isfinite(prob):
        #    print(ra, dec, prob)

        return prob

    def myloglike_circle(cube, ndim, nparams):
        ra = cube[0]
        dec = cube[1]

        ipix, radecs, patch, area = gwemopt.utils.getCirclePixels(ra, dec, config_struct["FOV"], nside)

        if len(ipix) == 0:
            prob = -np.inf
        else:
            prob = np.sum(map_struct_copy["prob"][ipix])

        if prob == 0:
            prob = -np.inf

        if np.isfinite(prob):
            print(ra, dec, prob)

        return prob

    plotDir = os.path.join(params["outputDir"],"multinest")
    if not os.path.isdir(plotDir):
        os.mkdir(plotDir)

    nside = params["nside"]
    tile_struct = {}
    for ii in range(Ntiles):

        parameters = ["ra","dec"]
        n_params = len(parameters)
        if config_struct["FOV_type"] == "square":
             pymultinest.run(myloglike_square, myprior, n_params, importance_nested_sampling = False, resume = True, verbose = False, sampling_efficiency = 'parameter', n_live_points = 1000, outputfiles_basename='%s/2-'%plotDir, evidence_tolerance = 0.05, multimodal = False, seed = 1)
        elif config_struct["FOV_type"] == "circle":
             pymultinest.run(myloglike_circle, myprior, n_params, importance_nested_sampling = False, resume = True, verbose = False, sampling_efficiency = 'parameter', n_live_points = 1000, outputfiles_basename='%s/2-'%plotDir, evidence_tolerance = 0.5, multimodal = False, seed = 1)

        multifile= os.path.join(plotDir,'2-.txt')
        data = np.loadtxt(multifile)
        loglikelihood = -(1/2.0)*data[:,1]
        idx = np.argmax(loglikelihood)
        ra_pointing = data[idx,2]
        dec_pointing = data[idx,3]

        tile_struct[ii] = {}

        if config_struct["FOV_type"] == "square":
            ipix, radecs, patch, area = gwemopt.utils.getSquarePixels(ra_pointing, dec_pointing, config_struct["FOV"], nside, alpha=0.8)
        elif config_struct["FOV_type"] == "circle":
            ipix, radecs, patch, area = gwemopt.utils.getCirclePixels(ra_pointing, dec_pointing, config_struct["FOV"], nside, alpha=0.8)

        tile_struct[ii]["ra"] = ra_pointing
        tile_struct[ii]["dec"] = dec_pointing
        tile_struct[ii]["ipix"] = ipix
        tile_struct[ii]["corners"] = radecs
        tile_struct[ii]["patch"] = patch
        tile_struct[ii]["area"] = area

        map_struct_copy["prob"][ipix] = 0.0
        os.system("rm %s/*"%plotDir)

    return tile_struct

def greedy_tiles_struct(params, config_struct, telescope, map_struct, Ntiles = 10):

    Ntiles = params["Ntiles"]

    map_struct_copy = copy.deepcopy(map_struct)
    skymapData = []
    skymapData.append(map_struct_copy["ra"])
    skymapData.append(map_struct_copy["dec"])
    skymapData.append(map_struct_copy["prob"])
    greedy_struct = PlaceTile(skymapData, config_struct, numtiles = Ntiles)
    tile_struct = greedy_struct.getSamples()

    return tile_struct   

def getRandomPos(ra, dec, nums=1):
    '''
    Return one or more random ra and dec from the list of ras and decs supplied
    pattern of return array [RA, RA, RA, Dec, Dec, Dec] for nums=3
    '''
    return np.hstack([np.random.choice(ra, nums), np.random.choice(dec, nums)])


class PlaceTile:
    def __init__(self, skymapData, config_struct, numtiles=2):
        self.skymapData = skymapData
        self.ra_map = self.skymapData[0]
        self.dec_map = self.skymapData[1]
        self.pVal = self.skymapData[2]
        self.config_struct = config_struct
        self.numtiles = numtiles

    def lnprior(self, skyposition): ### Basic uniform prior across the sky
        reshaped_skyposition = np.reshape(skyposition, (2, self.numtiles))
        ra_centers = reshaped_skyposition[0]
        dec_centers= reshaped_skyposition[1]

        ignore = (ra_centers > 360.) | (ra_centers < 0.) | (dec_centers > 90.) | (dec_centers < -90.)
        if np.sum(ignore):
            return -np.inf
        else:
            return 0
 

    def lnlikelihood(self, skyposition_cent):
        reshaped_skyposition = np.reshape(skyposition_cent, (2, self.numtiles))

        config_struct = self.config_struct
        pVal = copy.deepcopy(self.pVal)
        npix = len(pVal)
        nside = hp.npix2nside(npix) 

        ra_centers = reshaped_skyposition[0]
        dec_centers= reshaped_skyposition[1]
        probabilitySum = 0.0
        for ii in range(self.numtiles):
            if config_struct["FOV_type"] == "square":
                ipix, radecs, patch, area = gwemopt.utils.getSquarePixels(ra_centers[ii], dec_centers[ii], config_struct["FOV"], nside)
            elif config_struct["FOV_type"] == "circle":
                ipix, radecs, patch, area = gwemopt.utils.getCirclePixels(ra_centers[ii], dec_centers[ii], config_struct["FOV"], nside)

            prob = np.sum(pVal[ipix])
            pVal[ipix] = 0.0

            probabilitySum += prob

        return 4*np.log(probabilitySum)


    def lnpost(self, skypos):
        lp = self.lnprior(skypos)
        if not np.isfinite(lp):
            return -np.inf
        return lp + self.lnlikelihood(skypos)
        
        
    
    def getSamples(self):

        import emcee        

        config_struct = self.config_struct
        pVal = copy.copy(self.pVal)
        npix = len(pVal)
        nside = hp.npix2nside(npix)

        ndim, nwalkers = 2*self.numtiles, 4*self.numtiles
        include = np.cumsum(self.pVal) < 0.9
        #include[np.sum(include)] = True
        ra_included = self.ra_map[include]
        dec_included = self.dec_map[include]
        p0 = []
        [p0.append(getRandomPos(self.ra_map, self.dec_map, nums=self.numtiles)) for _ in range(nwalkers)]
                    
        sampler = emcee.EnsembleSampler(nwalkers, ndim, self.lnpost)
        start = time.time()
        pos, prob, state = sampler.run_mcmc(p0, 100)
        print('Burned in...')
        print('Time taken to burn in = ' + str(time.time() - start))
        sampler.reset()
        start = time.time()
        result = sampler.run_mcmc(pos, 1000)
        end = time.time()
        print('Acceptance fraction = ' + str(np.mean(sampler.acceptance_fraction[:])))
        print('Time taken to finish the MCMC = ' + str(end - start))
        tileCenters = sampler.flatchain
        samples    = tileCenters.copy()

        peak_ras, peak_decs = self.localizeTC(samples=samples)

        tile_struct = {}
        #for ii in range(self.numtiles):
        #    ra_pointing = samples[0,ii]
        #    dec_pointing = samples[0,ii+self.numtiles]
        for ii in range(len(peak_ras)):
            ra_pointing = peak_ras[ii]
            dec_pointing = peak_decs[ii]

            tile_struct[ii] = {}

            if config_struct["FOV_type"] == "square":
                ipix, radecs, patch, area = gwemopt.utils.getSquarePixels(ra_pointing, dec_pointing, config_struct["FOV"], nside, alpha=0.8)
            elif config_struct["FOV_type"] == "circle":
                ipix, radecs, patch, area = gwemopt.utils.getCirclePixels(ra_pointing, dec_pointing, config_struct["FOV"], nside, alpha=0.8)

        tile_struct[ii]["ra"] = ra_pointing
        tile_struct[ii]["dec"] = dec_pointing
        tile_struct[ii]["ipix"] = ipix
        tile_struct[ii]["corners"] = radecs
        tile_struct[ii]["patch"] = patch
        tile_struct[ii]["area"] = area

        return tile_struct
        
        
    def optimizeBins(self, ra, dec, masked_points=np.array([])):
        config_struct = self.config_struct

        pVal = copy.deepcopy(self.pVal)
        if len(masked_points) > 0:
            pVal[masked_points.astype(int)] = 0.0
        npix = len(pVal)
        nside = hp.npix2nside(npix)
 
        binTrials = int(np.sqrt(len(ra)))
        bins_array = np.arange(2, binTrials + 1)
        probs_allChosenTiles = np.array([])
    
        for bins in bins_array:

            [hist, ra_bin, dec_bin] = np.histogram2d(ra, dec, bins)
            ra_bin_cent = 0.5*(ra_bin[1:] + ra_bin[:-1])
            dec_bin_cent = 0.5*(dec_bin[1:] + dec_bin[:-1])
            max_pos =np.argmax(hist)
            x = int(np.argmax(hist)/bins)
            y = np.argmax(hist)%bins
            ra_peak = ra_bin_cent[x]
            dec_peak = dec_bin_cent[y]

            if config_struct["FOV_type"] == "square":
                ipix, radecs, patch, area = gwemopt.utils.getSquarePixels(ra_peak, dec_peak, config_struct["FOV"], nside)
            elif config_struct["FOV_type"] == "circle":
                ipix, radecs, patch, area = gwemopt.utils.getCirclePixels(ra_peak, dec_peak, config_struct["FOV"], nside)

            prob_encl = np.sum(pVal[ipix])
            probs_allChosenTiles = np.append(probs_allChosenTiles, prob_encl)
        
        bin_max = bins_array[np.argmax(probs_allChosenTiles)]

        [hist_max, ra_bin_max, dec_bin_max] = np.histogram2d(ra, dec, bin_max)
        ra_bin_cent = 0.5*(ra_bin_max[1:] + ra_bin_max[:-1])
        dec_bin_cent = 0.5*(dec_bin_max[1:] + dec_bin_max[:-1])

    
        y = np.argmax(hist_max)%bin_max
        x = int(np.argmax(hist_max)/bin_max)
        ra_peak = ra_bin_cent[x]
        dec_peak = dec_bin_cent[y]

        if config_struct["FOV_type"] == "square":
            ipix, radecs, patch, area = gwemopt.utils.getSquarePixels(ra_peak, dec_peak, config_struct["FOV"], nside)
        elif config_struct["FOV_type"] == "circle":
            ipix, radecs, patch, area = gwemopt.utils.getCirclePixels(ra_peak, dec_peak, config_struct["FOV"], nside)
    
        return ra_peak, dec_peak, ipix

    def localizeTC(self, reference=None, samples=None, verbose=False):
    
        if samples is None:
            samples = self.getSamples()
        else:
            print('\n\nReading data from pickled files...')

        ra_samples = []
        dec_samples = []
        for ii in range(0, self.numtiles):
            ra_samples.append(samples[:,ii])
            dec_samples.append(samples[:,ii+self.numtiles])

        masked_points=np.array([])
        probabilitySum = 0.0
        RA_Peak_list = []
        Dec_Peak_list = []
        for ii in range(self.numtiles):
            ra_peak, dec_peak, points_to_be_masked = self.optimizeBins(ra_samples[ii], dec_samples[ii], masked_points)

            masked_points = np.hstack((masked_points, points_to_be_masked))
            RA_Peak_list.append(ra_peak)
            Dec_Peak_list.append(dec_peak)

        return [RA_Peak_list, Dec_Peak_list]


    
    













import os, sys
import numpy as np
import healpy as hp
import copy

import astropy.coordinates
from astropy.time import Time, TimeDelta
import astropy.units as u
from joblib import Parallel, delayed
import ephem

import ligo.segments as segments
import gwemopt.utils

def get_telescope_segments(params):

    for telescope in params["telescopes"]:

        params["config"][telescope]["segmentlist"] = get_segments(params, params["config"][telescope])
        params["config"][telescope]["exposurelist"] = gwemopt.utils.get_exposures(params, params["config"][telescope], params["config"][telescope]["segmentlist"])

        nexp, junk = np.array(params["config"][telescope]["exposurelist"]).shape        
        params["config"][telescope]["n_windows"] = nexp
        tot_obs_time = np.sum(np.diff(np.array(params["config"][telescope]["exposurelist"]))) * 86400.
        params["config"][telescope]["tot_obs_time"] = tot_obs_time

    return params

def get_moon_segments(config_struct,segmentlist,observer,fxdbdy,radec):

    moonsegmentlist = segments.segmentlist()
    dt = 1.0/24.0
    tt = np.arange(segmentlist[0][0],segmentlist[-1][1]+dt,dt)

    ra2 = radec.ra.radian
    d2 = radec.dec.radian

    # Where is the moon?
    moon = ephem.Moon()
    for ii in range(len(tt)-1):
        observer.date = ephem.Date(Time(tt[ii], format='mjd', scale='utc').iso)
        moon.compute(observer)
        fxdbdy.compute(observer)

        alt_target = float(repr(fxdbdy.alt)) * (360/(2*np.pi))
        az_target = float(repr(fxdbdy.az)) * (360/(2*np.pi))
        #print("Altitude / Azimuth of target: %.5f / %.5f"%(alt_target,az_target))

        alt_moon = float(repr(moon.alt)) * (360/(2*np.pi))
        az_moon = float(repr(moon.az)) * (360/(2*np.pi))
        #print("Altitude / Azimuth of moon: %.5f / %.5f"%(alt_moon,az_moon))

        ra_moon = (180/np.pi)*float(repr(moon.ra))
        dec_moon = (180/np.pi)*float(repr(moon.dec))

        # Coverting both target and moon ra and dec to radians
        ra1 = float(repr(moon.ra))
        d1 = float(repr(moon.dec))

        # Calculate angle between target and moon
        cosA = np.sin(d1)*np.sin(d2) + np.cos(d1)*np.cos(d2)*np.cos(ra1-ra2)
        angle = np.arccos(cosA)*(360/(2*np.pi))
        #print("Angle between moon and target: %.5f"%(angle))

        #if angle >= 50.0*moon.moon_phase**2:
        if angle >= 20.0:
            segment = segments.segment(tt[ii],tt[ii+1])
            moonsegmentlist = moonsegmentlist + segments.segmentlist([segment])
            moonsegmentlist.coalesce()

    moonsegmentlistdic = segments.segmentlistdict()
    moonsegmentlistdic["observations"] = segmentlist
    moonsegmentlistdic["moon"] = moonsegmentlist
    moonsegmentlist = moonsegmentlistdic.intersection(["observations","moon"])
    moonsegmentlist.coalesce()

    return moonsegmentlist

def get_skybrightness(config_struct,segmentlist,observer,fxdbdy,radec):

    moonsegmentlist = segments.segmentlist()
    if config_struct["filt"] == "c":
        passband = "g"
    else:
        passband = config_struct["filt"]

    # Moon phase data (from Coughlin, Stubbs, and Claver Table 2) 
    moon_phases = [2,10,45,90]
    moon_data = {'u':[2.7,3.1,4.2,5.7],
                 'g':[2.4,2.8,3.8,5.2],
                 'r':[2.1,2.5,3.4,4.9],
                 'i':[1.9,2.3,3.3,4.7],
                 'z':[1.9,2.2,3.2,4.6],
                 'y':[1.8,2.2,3.1,4.5]}

    # Determine moon data for this phase
    moon_data_passband = moon_data[passband]

    # Fits to solar sky brightness (from Coughlin, Stubbs, and Claver Table 4) 
    sun_data = {'u':[88.5,-0.5,-0.5,0.4],
                'g':[386.5,-2.2,-2.4,0.8],
                'r':[189.0,-1.4,-1.1,0.8],
                'i':[164.8,-1.5,-0.7,0.6],
                'z':[231.2,-2.8,-0.7,1.4],
                'zs':[131.1,-1.4,-0.5,0.2],
                'y':[92.0,-1.3,-0.2,0.9]}

    sun_data_error = {'u':[6.2,0.1,0.1,0.1],
                'g':[34.0,0.2,0.2,0.5],
                'r':[32.7,0.2,0.2,0.5],
                'i':[33.1,0.2,0.2,0.5],
                'z':[62.3,0.3,0.4,0.9],
                'zs':[45.6,0.2,0.3,0.6],
                'y':[32.7,0.2,0.2,0.5]}

    # Determine sun data for this phase
    sun_data_passband = sun_data[passband]

    dt = 1.0/24.0
    tt = np.arange(segmentlist[0][0],segmentlist[-1][1]+dt,dt)

    ra2 = radec.ra.radian
    d2 = radec.dec.radian

    # Where is the moon?
    moon = ephem.Moon()
    for ii in range(len(tt)-1):
        observer.date = ephem.Date(Time(tt[ii], format='mjd', scale='utc').iso)
        moon.compute(observer)
        fxdbdy.compute(observer)

        alt_target = float(repr(fxdbdy.alt)) * (360/(2*np.pi))
        az_target = float(repr(fxdbdy.az)) * (360/(2*np.pi))
        #print("Altitude / Azimuth of target: %.5f / %.5f"%(alt_target,az_target))

        alt_moon = float(repr(moon.alt)) * (360/(2*np.pi))
        az_moon = float(repr(moon.az)) * (360/(2*np.pi))
        #print("Altitude / Azimuth of moon: %.5f / %.5f"%(alt_moon,az_moon))

        if (alt_target < 30.0) or (alt_moon < 30.0):
            total_mag, total_mag_error, flux_mag, flux_mag_error = np.inf, np.inf, np.inf, np.inf
        else:
            ra_moon = (180/np.pi)*float(repr(moon.ra))
            dec_moon = (180/np.pi)*float(repr(moon.dec))

            # Coverting both target and moon ra and dec to radians
            ra1 = float(repr(moon.ra))
            d1 = float(repr(moon.dec))

            # Calculate angle between target and moon
            cosA = np.sin(d1)*np.sin(d2) + np.cos(d1)*np.cos(d2)*np.cos(ra1-ra2)
            angle = np.arccos(cosA)*(360/(2*np.pi))
            #print("Angle between moon and target: %.5f"%(angle))

            delta_mag = np.interp(moon.moon_phase*100.0,moon_phases,moon_data_passband)
            delta_mag_error = 0.1*delta_mag

            flux = sun_data_passband[0] + sun_data_passband[1]*angle +\
                sun_data_passband[2]*alt_target + sun_data_passband[3]*alt_moon
            flux_zp = sun_data_passband[0] + sun_data_passband[1]*90.0 +\
                sun_data_passband[2]*90.0 + sun_data_passband[3]*45.0

            # check if flux < 0: too small to fit
            if flux < 0:
                flux = 1e-10

            flux = flux* (10**11)
            flux_zp = flux_zp* (10**11)
            flux_mag = -2.5 * (np.log10(flux) - np.log10(flux_zp))

            sun_data_passband_error = sun_data_error[passband]
            flux_error = np.sqrt(sun_data_passband_error[0]**2 + sun_data_passband_error[1]**2 * angle**2 +\
                sun_data_passband_error[2]**2 * alt_target**2 + sun_data_passband_error[3]**2 * alt_moon**2)
            flux_error = flux_error * (10**11)

            flux_mag_error = 1.08574 * flux_error / flux

            # Determine total magnitude contribution
            total_mag = delta_mag + flux_mag
            total_mag_error = np.sqrt(delta_mag_error**2 + flux_mag_error**2)
            #print(tt[ii], angle, alt_target, alt_moon, total_mag, total_mag_error)
        if total_mag > 0.0:
            segment = segments.segment(tt[ii],tt[ii+1])
            moonsegmentlist = moonsegmentlist + segments.segmentlist([segment])
            moonsegmentlist.coalesce()
        #else:
        #    print(tt[ii], angle, alt_target, alt_moon, total_mag, total_mag_error)

    moonsegmentlistdic = segments.segmentlistdict()
    moonsegmentlistdic["observations"] = segmentlist
    moonsegmentlistdic["moon"] = moonsegmentlist
    moonsegmentlist = moonsegmentlistdic.intersection(["observations","moon"])
    moonsegmentlist.coalesce()

    #print("Keeping %.2f %% of data"%(100.0*np.sum(np.diff(moonsegmentlist))/np.sum(np.diff(segmentlist))))

    return moonsegmentlist

def get_segments(params, config_struct):

    gpstime = params["gpstime"]
    event_mjd = Time(gpstime, format='gps', scale='utc').mjd

    segmentlist = segments.segmentlist()
    n_windows = len(params["Tobs"]) // 2
    start_segments = event_mjd + params["Tobs"][::2]
    end_segments = event_mjd + params["Tobs"][1::2]
    for start_segment, end_segment in zip(start_segments,end_segments):
        segmentlist.append(segments.segment(start_segment,end_segment))

    observer = ephem.Observer()
    observer.lat = str(config_struct["latitude"])
    observer.lon = str(config_struct["longitude"])
    observer.horizon = str(-12.0)
    observer.elevation = config_struct["elevation"]

    date_start = ephem.Date(Time(segmentlist[0][0], format='mjd', scale='utc').iso)
    date_end = ephem.Date(Time(segmentlist[-1][1], format='mjd', scale='utc').iso)
    observer.date = ephem.Date(Time(segmentlist[0][0], format='mjd', scale='utc').iso)

    sun = ephem.Sun()
    nightsegmentlist = segments.segmentlist()
    while date_start < date_end:
        date_rise = observer.next_rising(sun, start = date_start)
        date_set = observer.next_setting(sun, start = date_start)
        if date_set > date_rise:
            date_set = observer.previous_setting(sun, start = date_start)

        astropy_rise = Time(date_rise.datetime(), scale='utc').mjd
        astropy_set  = Time(date_set.datetime(), scale='utc').mjd

        segment = segments.segment(astropy_set,astropy_rise)
        nightsegmentlist = nightsegmentlist + segments.segmentlist([segment])
        nightsegmentlist.coalesce()

        date_start = date_rise
        observer.date = date_rise

    segmentlistdic = segments.segmentlistdict()
    segmentlistdic["observations"] = segmentlist
    segmentlistdic["night"] = nightsegmentlist
    segmentlist = segmentlistdic.intersection(["observations","night"])
    segmentlist.coalesce()

    return segmentlist

def get_segments_tile(config_struct, observatory, radec, segmentlist):

    observer = ephem.Observer()
    observer.lat = str(config_struct["latitude"])
    observer.lon = str(config_struct["longitude"])
    observer.horizon = str(30.0)
    observer.elevation = config_struct["elevation"]

    fxdbdy = ephem.FixedBody()
    fxdbdy._ra = ephem.degrees(str(radec.ra.degree))
    fxdbdy._dec = ephem.degrees(str(radec.dec.degree))

    observer.date = ephem.Date(Time(segmentlist[0][0], format='mjd', scale='utc').iso)
    fxdbdy.compute(observer)

    date_start = ephem.Date(Time(segmentlist[0][0], format='mjd', scale='utc').iso)
    date_end = ephem.Date(Time(segmentlist[-1][1], format='mjd', scale='utc').iso)
    tilesegmentlist = segments.segmentlist()
    while date_start < date_end:
        try:
            date_rise = observer.next_rising(fxdbdy, start=observer.date)
            date_set = observer.next_setting(fxdbdy, start=observer.date)
            if date_rise > date_set:
                date_rise = observer.previous_rising(fxdbdy, start=observer.date)
        except ephem.AlwaysUpError:
            date_rise = date_start
            date_set = date_end
        except ephem.NeverUpError:
            date_rise = ephem.Date(0.0)
            date_set = ephem.Date(0.0)
            break

        astropy_rise = Time(date_rise.datetime(), scale='utc')
        astropy_set  = Time(date_set.datetime(), scale='utc')

        astropy_rise_mjd = astropy_rise.mjd
        astropy_set_mjd  = astropy_set.mjd
        # Alt/az reference frame at observatory, now
        #frame_rise = astropy.coordinates.AltAz(obstime=astropy_rise, location=observatory)
        #frame_set = astropy.coordinates.AltAz(obstime=astropy_set, location=observatory)    
        # Transform grid to alt/az coordinates at observatory, now
        #altaz_rise = radec.transform_to(frame_rise)
        #altaz_set = radec.transform_to(frame_set)        

        segment = segments.segment(astropy_rise_mjd,astropy_set_mjd)
        tilesegmentlist = tilesegmentlist + segments.segmentlist([segment])
        tilesegmentlist.coalesce()

        date_start = date_set
        observer.date = date_set

    #moonsegmentlist = get_skybrightness(\
    #    config_struct,segmentlist,observer,fxdbdy,radec)

    moonsegmentlist = get_moon_segments(\
        config_struct,segmentlist,observer,fxdbdy,radec)

    tilesegmentlistdic = segments.segmentlistdict()
    tilesegmentlistdic["observations"] = segmentlist
    tilesegmentlistdic["tile"] = tilesegmentlist
    tilesegmentlistdic["moon"] = moonsegmentlist
    tilesegmentlist = tilesegmentlistdic.intersection(["observations","tile","moon"])
    #tilesegmentlist = tilesegmentlistdic.intersection(["observations","tile"])
    tilesegmentlist.coalesce()

    return tilesegmentlist

def get_segments_tiles(params, config_struct, tile_struct):

    observatory = astropy.coordinates.EarthLocation(
        lat=config_struct["latitude"]*u.deg, lon=config_struct["longitude"]*u.deg, height=config_struct["elevation"]*u.m)

    segmentlist = config_struct["segmentlist"]

    print("Generating segments for tiles...")

    ras = []
    decs = []
    keys = tile_struct.keys()
    for key in keys:
        ras.append(tile_struct[key]["ra"])
        decs.append(tile_struct[key]["dec"])

    # Convert to RA, Dec.
    radecs = astropy.coordinates.SkyCoord(
            ra=np.array(ras)*u.degree, dec=np.array(decs)*u.degree, frame='icrs')

    if params["doParallel"]:
        tilesegmentlists = Parallel(n_jobs=params["Ncores"])(delayed(get_segments_tile)(config_struct, observatory, radec, segmentlist) for radec in radecs)
        for ii,key in enumerate(keys):
            tile_struct[key]["segmentlist"] = tilesegmentlists[ii]
    else:
        for ii,key in enumerate(keys):
            #if np.mod(ii,100) == 0: 
            #    print("Generating segments for tile %d/%d"%(ii+1,len(radecs)))
            radec = radecs[ii]
            tilesegmentlist = get_segments_tile(config_struct, observatory, radec, segmentlist)
            tile_struct[key]["segmentlist"] = tilesegmentlist

    return tile_struct




import os, sys
import copy

import numpy as np
import healpy as hp

from scipy.stats import norm

from astropy.time import Time

import gwemopt.utils
import gwemopt.rankedTilesGenerator
import gwemopt.samplers, gwemopt.segments
import gwemopt.quadrants
import gwemopt.moc

def galaxy(params, map_struct, catalog_struct):

    nside = params["nside"]

    tile_structs = {}
    for telescope in params["telescopes"]:
        config_struct = params["config"][telescope]

        moc_struct = {}
        cnt = 0
        for ra, dec, Sloc, S in zip(catalog_struct["ra"], catalog_struct["dec"], catalog_struct["Sloc"], catalog_struct["S"]):
            moc_struct[cnt] = gwemopt.moc.Fov2Moc(params, config_struct, telescope, ra, dec, nside)
            cnt = cnt + 1

        tile_struct = powerlaw_tiles_struct(params, config_struct, telescope, map_struct, moc_struct)
        tile_struct = gwemopt.segments.get_segments_tiles(params, config_struct, tile_struct)

        cnt = 0
        for ra, dec, Sloc, S in zip(catalog_struct["ra"], catalog_struct["dec"], catalog_struct["Sloc"], catalog_struct["S"]):
            tile_struct[cnt]['prob'] = Sloc
            cnt = cnt + 1

        tile_structs[telescope] = tile_struct

    return tile_structs

def greedy(params, map_struct):

    tile_structs = {}
    for telescope in params["telescopes"]:
        config_struct = params["config"][telescope]

        tile_struct = gwemopt.samplers.greedy_tiles_struct(params, config_struct, telescope, map_struct)
        tile_struct = gwemopt.segments.get_segments_tiles(params, config_struct, tile_struct)

        tile_structs[telescope] = tile_struct

    return tile_structs

def hierarchical(params, map_struct):

    tile_structs = {}
    for telescope in params["telescopes"]:
        config_struct = params["config"][telescope]

        tile_struct = gwemopt.samplers.hierarchical_tiles_struct(params, config_struct, telescope, map_struct)
        tile_struct = gwemopt.segments.get_segments_tiles(params, config_struct, tile_struct)

        tile_structs[telescope] = tile_struct

    return tile_structs

def powerlaw_tiles_struct(params, config_struct, telescope, map_struct, tile_struct):

    tot_obs_time = config_struct["tot_obs_time"]

    if "observability" in map_struct:
        prob = map_struct["observability"][telescope]["prob"]
    else:
        prob = map_struct["prob"]

    n, cl, dist_exp = params["powerlaw_n"], params["powerlaw_cl"], params["powerlaw_dist_exp"]
    tile_probs = compute_tiles_map(tile_struct, prob, func='np.sum(x)')
    tile_probs[tile_probs<np.max(tile_probs)*0.01] = 0.0 
    prob_scaled = copy.deepcopy(prob)
    prob_sorted = np.sort(prob_scaled)[::-1]
    prob_indexes = np.argsort(prob_scaled)[::-1]
    prob_cumsum = np.cumsum(prob_sorted)
    index = np.argmin(np.abs(prob_cumsum - cl)) + 1
    prob_scaled[prob_indexes[index:]] = 0.0
    prob_scaled = prob_scaled**n
    prob_scaled = prob_scaled / np.nansum(prob_scaled)

    ranked_tile_probs = compute_tiles_map(tile_struct, prob_scaled, func='np.sum(x)')
    ranked_tile_probs[np.isnan(ranked_tile_probs)] = 0.0
    ranked_tile_probs_thresh = np.max(ranked_tile_probs)*0.01
    ranked_tile_probs[ranked_tile_probs<=ranked_tile_probs_thresh] = 0.0
    ranked_tile_probs = ranked_tile_probs / np.nansum(ranked_tile_probs)

    if "distmed" in map_struct:
        distmed = map_struct["distmed"]
        distmed[distmed<=0] = np.nan
        distmed[~np.isfinite(distmed)] = np.nan
        #distmed[distmed<np.nanmedian(distmed)/4.0] = np.nanmedian(distmed)/4.0

        ranked_tile_distances = compute_tiles_map(tile_struct, distmed, func='np.nanmedian(x)')        
        ranked_tile_distances_median = ranked_tile_distances / np.nanmedian(ranked_tile_distances)
        ranked_tile_distances_median = ranked_tile_distances_median**dist_exp
        ranked_tile_probs = ranked_tile_probs*ranked_tile_distances_median
        ranked_tile_probs = ranked_tile_probs / np.nansum(ranked_tile_probs)
        ranked_tile_probs[np.isnan(ranked_tile_probs)] = 0.0

    if params["doSingleExposure"]:
        keys = tile_struct.keys()
        ranked_tile_times = np.zeros((len(ranked_tile_probs),len(params["exposuretimes"])))
        for ii in range(len(params["exposuretimes"])):
            ranked_tile_times[ranked_tile_probs>0,ii] = params["exposuretimes"][ii]
        for key, prob, exposureTime, tileprob in zip(keys, ranked_tile_probs, ranked_tile_times, tile_probs):
            tile_struct[key]["prob"] = tileprob
            if prob == 0.0:
                tile_struct[key]["exposureTime"] = 0.0
                tile_struct[key]["nexposures"] = 0
                tile_struct[key]["filt"] = []
            else:
                if params["doReferences"]:
                    tile_struct[key]["exposureTime"] = []
                    tile_struct[key]["nexposures"] = []
                    tile_struct[key]["filt"] = []
                    if key in config_struct["reference_images"]:
                        for ii in range(len(params["filters"])):
                            if params["filters"][ii] in config_struct["reference_images"][key]:
                                tile_struct[key]["exposureTime"].append(exposureTime[ii])
                                tile_struct[key]["filt"].append(params["filters"][ii])
                        tile_struct[key]["nexposures"] = len(tile_struct[key]["exposureTime"])
                    else:
                        tile_struct[key]["exposureTime"] = 0.0
                        tile_struct[key]["nexposures"] = 0
                        tile_struct[key]["filt"] = []
                else:
                    tile_struct[key]["exposureTime"] = exposureTime
                    tile_struct[key]["nexposures"] = len(params["exposuretimes"])
                    tile_struct[key]["filt"] = params["filters"]

    else:
        ranked_tile_times = gwemopt.utils.integrationTime(tot_obs_time, ranked_tile_probs, func=None, T_int=config_struct["exposuretime"])

        keys = tile_struct.keys()
        for key, prob, exposureTime, tileprob in zip(keys, ranked_tile_probs, ranked_tile_times, tile_probs):
            tile_struct[key]["prob"] = prob
            tile_struct[key]["exposureTime"] = exposureTime
            tile_struct[key]["nexposures"] = int(np.floor(exposureTime/config_struct["exposuretime"]))
            tile_struct[key]["filt"] = [config_struct["filt"]] * tile_struct[key]["nexposures"]

    return tile_struct

def moc(params, map_struct, moc_structs):

    tile_structs = {}
    for telescope in params["telescopes"]:
 
        config_struct = params["config"][telescope]
        moc_struct = moc_structs[telescope]
 
        tile_struct = powerlaw_tiles_struct(params, config_struct, telescope, map_struct, moc_struct)

        tile_struct = gwemopt.segments.get_segments_tiles(params, config_struct, tile_struct)
        tile_structs[telescope] = tile_struct

    return tile_structs

def pem_tiles_struct(params, config_struct, telescope, map_struct, tile_struct):

    tot_obs_time = config_struct["tot_obs_time"]

    if "observability" in map_struct:
        prob = map_struct["observability"][telescope]["prob"]
    else:
        prob = map_struct["prob"]

    ranked_tile_probs = compute_tiles_map(tile_struct, prob, func='np.sum(x)')
    ranked_tile_times = gwemopt.utils.integrationTime(tot_obs_time, ranked_tile_probs, func=None, T_int=config_struct["exposuretime"])

    lim_mag = config_struct["magnitude"]
    lim_time = config_struct["exposuretime"]

    tau = np.arange(lim_time,3600.0,lim_time)
    Loftau = None

    N_ref = 9.7847e9
    L_min = 4.9370e31
    L_max = 4.9370e33

    if "distmu" in map_struct:
        prob = map_struct["prob"]
        distnorm = map_struct["distnorm"]
        distmu = map_struct["distmu"]
        distsigma = map_struct["distsigma"]

        D_min = 1.0e7
        D_max = 1.0e10
        R = np.linspace(D_min/1e6, D_max/1e6)
        p_R = [np.sum(prob * rr**2 * distnorm * norm(distmu, distsigma).pdf(rr)) for rr in R]
        p_R = np.array(p_R)
        p_R = p_R / len(prob)

        R = R*1e6
        D_mu = None
        D_sig = None
    else:
        D_mu = 200.0e6
        D_sig = 60.0e6
        R = None
        p_R = None

    tau, prob = gwemopt.pem.Pem(lim_mag, lim_time, N_ref = N_ref, L_min = L_min, L_max = L_max, tau = tau, Loftau = Loftau, D_mu = D_mu, D_sig = D_sig, R = R, p_R = p_R)

    tprob, time_allocation = gwemopt.pem.Main(tot_obs_time, 0, 0, ranked_tile_probs, tau, prob, 'Eq')

    if params["doPlots"]:
        gwemopt.plotting.tauprob(params,tau,prob)

    keys = tile_struct.keys()
    for key, prob, exposureTime in zip(keys, ranked_tile_probs, time_allocation):
        tile_struct[key]["prob"] = prob
        tile_struct[key]["exposureTime"] = exposureTime
        tile_struct[key]["nexposures"] = int(np.floor(exposureTime/config_struct["exposuretime"]))
        tile_struct[key]["filt"] = [config_struct["filt"]] * tile_struct[key]["nexposures"]

    return tile_struct

def compute_tiles_map(tile_struct, skymap, func=None):

    if func is None:
        f = lambda x: np.sum(x)
    else:
        f = lambda x: eval(func)

    keys = tile_struct.keys()
    ntiles = len(keys)
    vals = np.nan*np.ones((ntiles,))
    for ii,key in enumerate(tile_struct.keys()):
        vals[ii] = f(skymap[tile_struct[key]["ipix"]])

    return vals

def tesselation_spiral(config_struct):
    if config_struct["FOV_type"] == "square":
        FOV = config_struct["FOV"]*config_struct["FOV"]
    elif config_struct["FOV_type"] == "circle":
        FOV = np.pi*config_struct["FOV"]*config_struct["FOV"]

    area_of_sphere = 4*np.pi*(180/np.pi)**2
    n = int(np.ceil(area_of_sphere/FOV))
    print("Using %d points to tile the sphere..."%n)
 
    golden_angle = np.pi * (3 - np.sqrt(5))
    theta = golden_angle * np.arange(n)
    z = np.linspace(1 - 1.0 / n, 1.0 / n - 1, n)
    radius = np.sqrt(1 - z * z)
 
    points = np.zeros((n, 3))
    points[:,0] = radius * np.cos(theta)
    points[:,1] = radius * np.sin(theta)
    points[:,2] = z

    ra, dec = hp.pixelfunc.vec2ang(points, lonlat=True)
    fid = open(config_struct["tesselationFile"],'w')
    for ii in range(len(ra)):
        fid.write('%d %.5f %.5f\n'%(ii,ra[ii],dec[ii]))
    fid.close()   

def tesselation_packing(config_struct):
    sphere_radius = 1.0
    if config_struct["FOV_type"] == "square":
        circle_radius = np.deg2rad(config_struct["FOV"]/2.0)
    elif config_struct["FOV_type"] == "circle":
        circle_radius = np.deg2rad(config_struct["FOV"])
    vertical_count = int( (np.pi*sphere_radius)/(2*circle_radius) )

    phis = []
    thetas = []

    phi = -0.5*np.pi
    phi_step = np.pi/vertical_count
    while phi<0.5*np.pi:
        horizontal_count = int( (2*np.pi*np.cos(phi)*sphere_radius)/(2*circle_radius) )
        if horizontal_count==0: horizontal_count=1
        theta = 0
        theta_step = 2*np.pi/horizontal_count
        while theta<2*np.pi-1e-8:
            phis.append(phi)
            thetas.append(theta)
            theta += theta_step
        phi += phi_step
    dec = np.array(np.rad2deg(phis))
    ra = np.array(np.rad2deg(thetas))

    fid = open(config_struct["tesselationFile"],'w')
    for ii in range(len(ra)):
        fid.write('%d %.5f %.5f\n'%(ii,ra[ii],dec[ii]))
    fid.close()





import h5py
import astropy.io.fits

filename = '../catalog/CLU_20170106_galexwise_DaveUpdate.fits'
filename = '../catalog/CLU_20181213V2.fits'

hdul = astropy.io.fits.open(filename)
data = hdul[1].data

names = hdul[1].data.name
ras, decs = hdul[1].data.ra, hdul[1].data.dec
sfr, mstar = hdul[1].data.sfr_fuv, hdul[1].data.mstar
distmpc, magb = hdul[1].data.distmpc, hdul[1].data.magb
a, b2a, pa = hdul[1].data.a, hdul[1].data.b2a, hdul[1].data.pa
btc = hdul[1].data.btc

with h5py.File('../catalog/CLU.hdf5', 'w') as f:
    f.create_dataset('name', data=names)
    f.create_dataset('ra', data=ras)
    f.create_dataset('dec', data=decs)
    f.create_dataset('sfr_fuv', data=sfr)
    f.create_dataset('mstar', data=mstar)
    f.create_dataset('distmpc', data=distmpc)
    f.create_dataset('magb', data=magb)
    f.create_dataset('a', data=a)
    f.create_dataset('b2a', data=b2a)
    f.create_dataset('pa', data=pa)
    f.create_dataset('btc', data=btc)



# -*- coding: utf-8 -*-
# Copyright (C) Michael Coughlin and Christopher Stubbs(2015)
#
# skybrightness is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# gwemopt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with skybrightness.  If not, see <http://www.gnu.org/licenses/>.

"""This module provides example methods to calculate the optimal likelihood and distance scalings for optimizing telescope time allocations.
"""

import os, sys, glob, optparse
import numpy as np

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
matplotlib.rcParams.update({
        "font.size": 24.0,
        "axes.titlesize": 24.0,
        "axes.labelsize": 24.0,
        "xtick.labelsize": 24.0,
        "ytick.labelsize": 24.0,
        "legend.fontsize": 24.0,
        "figure.dpi": 600,
        "savefig.dpi": 600,
        "text.usetex": True
})

__author__  = "Michael Coughlin <michael.coughlin@ligo.org>"
__author__  = "Duncan Meacher <duncan.meacher@ligo.org>"
__date__    = "2016/01/06"
__version__ = "0.1"

def parse_commandline():
    """@parse the options given on the command-line.
    """
    parser = optparse.OptionParser(usage=__doc__,version=__version__)

    parser.add_option("-o","--outputDir",help="Output location",default = '/home/mcoughlin/Skymaps/optimization/plots')

    parser.add_option("--minabsm", help="Min absolute magnitude.",default=-16,type=float)
    parser.add_option("--maxabsm", help="Max absolute magnitude.",default=-10,type=float)
    parser.add_option("--minfade", help="Min time above detection threshold [hours].",default=0.1,type=float)
    parser.add_option("--maxfade", help="Max time above detection threshold [hours].",default=10,type=float)

    parser.add_option("-v", "--verbose", action="store_true", default=False,help="Run verbosely. (Default: False)")

    opts, args = parser.parse_args()

    # show parameters
    if opts.verbose:
        print >> sys.stderr, ""
        print >> sys.stderr, "running dL.py..."
        print >> sys.stderr, "version: %s"%__version__
        print >> sys.stderr, ""
        print >> sys.stderr, "***************** PARAMETERS ********************"
        for o in opts.__dict__.items():
          print >> sys.stderr, o[0]+":"
          print >> sys.stderr, o[1]
        print >> sys.stderr, ""

    return opts

def combine_params(runpath):
    folders = glob.glob(os.path.join(runpath,"*"))
    for ii,folder in enumerate(folders):
        #baseDir = os.path.join(folder,"-16--10")
        baseDir = "%s/%d-%d/%.2f-%.2f"%(folder,minabsm,maxabsm,minfade,maxfade)
        #resultsfile = os.path.join(baseDir,"best.dat")
        resultsfile = os.path.join(baseDir,"samples.dat")
        if not os.path.isfile(resultsfile): continue
        data_out = np.loadtxt(resultsfile)
        if ii == 0:
            params = data_out
        else:
            params = np.vstack((params,data_out))
    return params

def combine_results(runpath):

    folders = glob.glob(os.path.join(runpath,"*"))
    for ii,folder in enumerate(folders):
        #baseDir = os.path.join(folder,"-16--10")
        baseDir = "%s/%d-%d/%.2f-%.2f"%(folder,minabsm,maxabsm,minfade,maxfade)

        resultsfile = os.path.join(baseDir,"images.dat")
        if not os.path.isfile(resultsfile): continue
        nums_cumsum, image_array = load_results(baseDir)
        if ii == 0:
            image_arrays = image_array
        else:
            image_arrays = np.vstack((image_arrays,image_array))
    image_arrays = image_arrays.T
    for ii,col in enumerate(image_arrays):
        image_arrays[ii,:] = np.sort(col)

    idx_10 = int(np.floor(0.10*len(col)))
    idx_50 = int(np.floor(0.50*len(col)))
    idx_90 = int(np.floor(0.90*len(col)))

    image_10 = image_arrays[:,idx_10]
    image_50 = image_arrays[:,idx_50]
    image_90 = image_arrays[:,idx_90]

    times_interp = np.logspace(0,6,1000)
    image_10_interp = np.interp(times_interp,image_10,nums_cumsum)
    image_50_interp = np.interp(times_interp,image_50,nums_cumsum)
    image_90_interp = np.interp(times_interp,image_90,nums_cumsum)

    return nums_cumsum,image_10,image_50,image_90,times_interp,image_10_interp,image_50_interp,image_90_interp

def load_results(baseDir):

    resultsfile = os.path.join(baseDir,"images.dat")
    data_out = np.loadtxt(resultsfile)
    nums_cumsum = data_out[:,0]
    image_optimal_array_sorted = data_out[:,2] 

    nums_cumsum_all = np.linspace(0.0,1.0,1000)
 
    image_array = np.interp(nums_cumsum_all,nums_cumsum,image_optimal_array_sorted)
    image_array[np.isnan(image_array)] = np.inf

    return nums_cumsum_all, image_array

def hist_results(samples):

    bins = np.linspace(np.min(samples),np.max(samples),50)
    hist1, bin_edges = np.histogram(samples, bins=bins)
    hist1 = hist1 / float(np.sum(hist1))
    bins = (bins[1:] + bins[:-1])/2.0

    return bins, hist1

# =============================================================================
#
#                                    MAIN
#
# =============================================================================

# Parse command line
opts = parse_commandline()

minabsm = opts.minabsm
maxabsm = opts.maxabsm

minfade = opts.minfade
maxfade = opts.maxfade

outputDir = opts.outputDir 
resultsDir = os.path.join(outputDir,'results')
if not os.path.isdir(resultsDir):
    os.mkdir(resultsDir)

runpath = os.path.join(outputDir,'PS1')
nums_cumsum,ps1_10,ps1_50,ps1_90,times_interp,ps1_10_interp,ps1_50_interp,ps1_90_interp = combine_results(runpath)
ps1_params = combine_params(runpath)
runpath = os.path.join(outputDir,'ATLAS')
nums_cumsum,atlas_10,atlas_50,atlas_90,times_interp,atlas_10_interp,atlas_50_interp,atlas_90_interp = combine_results(runpath)
atlas_params = combine_params(runpath)
runpath = os.path.join(outputDir,'combined')
nums_cumsum,combined_10,combined_50,combined_90,times_interp,combined_10_interp,combined_50_interp,combined_90_interp = combine_results(runpath)
combined_params = combine_params(runpath)

plt.figure(figsize=(12,10))
plt.semilogx(ps1_50,100*nums_cumsum,'g--',label='PS1')
plt.semilogx(atlas_50,100*nums_cumsum,'r.-',label='ATLAS')
plt.semilogx(combined_50,100*nums_cumsum,'c',label='PS1/ATLAS')
plt.fill_between(times_interp,100*ps1_90_interp,100*ps1_10_interp,facecolor='green',alpha=0.5)
plt.fill_between(times_interp,100*atlas_90_interp,100*atlas_10_interp,facecolor='red',alpha=0.5)
plt.fill_between(times_interp,100*combined_90_interp,100*combined_10_interp,facecolor='cyan',alpha=0.5)
#plt.semilogx(ps1_10,100*nums_cumsum,'k--')
#plt.semilogx(ps1_90,100*nums_cumsum,'k--')
#plt.semilogx(atlas_10,100*nums_cumsum,'r.-')
#plt.semilogx(atlas_90,100*nums_cumsum,'r.-')
#plt.semilogx(combined_10,100*nums_cumsum,'c')
#plt.semilogx(combined_90,100*nums_cumsum,'c')
plt.legend(loc=2)
plt.xlim([10,1e6])
plt.ylim([0,100.0])
plt.xlabel('Time [s]')
plt.ylabel('Percentage of imaged counterparts')
plt.show()
plotName = os.path.join(resultsDir,'optimization.pdf')
plt.savefig(plotName,dpi=200)
plt.close('all')

plt.figure(figsize=(12,10))
bins1, hist1 = hist_results(ps1_params[:,0])
bins2, hist2 = hist_results(combined_params[:,0])
bins3, hist3 = hist_results(atlas_params[:,0])
bins4, hist4 = hist_results(combined_params[:,3])
#plt.plot(bins1, np.cumsum(hist1),'k',label='PS1')
#plt.plot(bins2, np.cumsum(hist2),'k--',label='PS1 with ATLAS')
#plt.plot(bins3, np.cumsum(hist3),'r',label='ATLAS')
#plt.plot(bins4, np.cumsum(hist4),'r--',label='ATLAS with PS1')
plt.plot(bins1, hist1,'k',label='PS1')
plt.plot(bins2, hist2,'k--',label='PS1 with ATLAS')
plt.plot(bins3, hist3,'r',label='ATLAS')
plt.plot(bins4, hist4,'r--',label='ATLAS with PS1')
plt.legend(loc='best')
#plt.xlim([10,1e6])
#plt.ylim([0,1.0])
plt.xlabel('Likelihood Powerlaw Index')
plt.ylabel('Probability Density Function')
plt.show()
plotName = os.path.join(resultsDir,'n.pdf')
plt.savefig(plotName,dpi=200)
plt.close('all')

plt.figure(figsize=(12,10))
bins1, hist1 = hist_results(ps1_params[:,1])
bins2, hist2 = hist_results(combined_params[:,1])
bins3, hist3 = hist_results(atlas_params[:,1])
bins4, hist4 = hist_results(combined_params[:,4])
#plt.plot(bins1, np.cumsum(hist1),'k',label='PS1')
#plt.plot(bins2, np.cumsum(hist2),'k--',label='PS1 with ATLAS')
#plt.plot(bins3, np.cumsum(hist3),'r',label='ATLAS')
#plt.plot(bins4, np.cumsum(hist4),'r--',label='ATLAS with PS1')
plt.plot(bins1, hist1,'k',label='PS1')
plt.plot(bins2, hist2,'k--',label='PS1 with ATLAS')
plt.plot(bins3, hist3,'r',label='ATLAS')
plt.plot(bins4, hist4,'r--',label='ATLAS with PS1')
plt.legend(loc='best')
#plt.xlim([10,1e6])
#plt.ylim([0,1.0])
plt.xlabel('Likelihood Confidence Level')
plt.ylabel('Probability Density Function')
plt.show()
plotName = os.path.join(resultsDir,'cl.pdf')
plt.savefig(plotName,dpi=200)
plt.close('all')

plt.figure(figsize=(12,10))
bins1, hist1 = hist_results(ps1_params[:,2])
bins2, hist2 = hist_results(combined_params[:,2])
bins3, hist3 = hist_results(atlas_params[:,2])
bins4, hist4 = hist_results(combined_params[:,5])
#plt.plot(bins1, np.cumsum(hist1),'k',label='PS1')
#plt.plot(bins2, np.cumsum(hist2),'k--',label='PS1 with ATLAS')
#plt.plot(bins3, np.cumsum(hist3),'r',label='ATLAS')
#plt.plot(bins4, np.cumsum(hist4),'r--',label='ATLAS with PS1')
plt.plot(bins1, hist1,'k',label='PS1')
plt.plot(bins2, hist2,'k--',label='PS1 with ATLAS')
plt.plot(bins3, hist3,'r',label='ATLAS')
plt.plot(bins4, hist4,'r--',label='ATLAS with PS1')
plt.legend(loc='best')
#plt.xlim([10,1e6])
#plt.ylim([0,1.0])
plt.xlabel('Distance Powerlaw Index')
plt.ylabel('Probability Density Function')
plt.show()
plotName = os.path.join(resultsDir,'r.pdf')
plt.savefig(plotName,dpi=200)
plt.close('all')
